"""
    This is part of Kerykeion (C) 2020 Giacomo Battaglia
"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from kerykeion.main import KrInstance
import swisseph as swe

# General Costants
ORBIT_ROUND = 1

CONJUCTION_VALUE = 10
OPPOSITION_VALUE = 8
SQUARE_VALUE = 5
TRIGON_VALUE = 7


class DiscepoloNumber():
    """
    Calculates the synastry using Ciro Discepolo's algorithm.
    Args: first user object, second user object
    """

    def __init__(self, user_a: KrInstance, user_b: KrInstance):
        self.user_a = user_a
        self.user_b = user_b
        user_a.get_all()
        user_b.get_all()
        self.first_points_list = user_a.planets_list[:7] + [user_a.houses_list[0],
                                                            user_a.houses_list[9]]
        self.second_points_list = user_b.planets_list[:7] + [user_b.houses_list[0],
                                                             user_b.houses_list[9]]

    def synastry_aspect(self, user_a, pl_us1, user_b, pl_us2):
        """ 
        Calculates the aspects between the 2 users.
        Args: first user object, first list of planets and houses,
        second user objcet, second list of planets and houses. 
        """

        distance = abs(swe.difdeg2n(self.first_points_list[pl_us1]["abs_pos"],
                                    self.second_points_list[pl_us2]["abs_pos"]))
        if (int(distance) <= 15) and ((self.first_points_list[pl_us1]["name"] == "Sun" and self.first_points_list[pl_us2]["name"] == "Sun")
                                      or (self.first_points_list[pl_us1]["name"] == "Moon" and self.first_points_list[pl_us2]["name"] == "Moon")
                                      or (self.first_points_list[pl_us1]["name"] == "Moon" and self.first_points_list[pl_us2]["name"] == "Sun")
                                      or (self.first_points_list[pl_us1]["name"] == "Sun" and self.first_points_list[pl_us2]["name"] == "Moon")):
            aspect = "Conjuction"
            return True, aspect, distance
        if int(distance) <= CONJUCTION_VALUE:
            aspect = "Conjuction"
            return True, aspect, distance
        elif (180 - OPPOSITION_VALUE) <= int(distance) <= (180 + OPPOSITION_VALUE):
            aspect = "Oposition"
            return True, aspect, distance - 180
        elif (90 - SQUARE_VALUE) <= int(distance) <= (90 + SQUARE_VALUE):
            aspect = "Square"
            return True, aspect, distance - 90
        elif (120 - TRIGON_VALUE) <= int(distance) <= (120 + TRIGON_VALUE):
            aspect = "Trigon"
            return True, aspect, distance - 120
        elif 57 <= int(distance) <= 63:
            aspect = "Sextil"
            return True, aspect, distance - 60
        elif 28 <= int(distance) <= 32:
            aspect = "Semisextil"
            return True, aspect, distance - 30
        elif 43 <= int(distance) <= 47:
            aspect = "Semisquare"
            return True, aspect, distance - 45
        elif 133 <= int(distance) <= 137:
            aspect = "Sesquiquadrate"
            return True, aspect, distance - 135
        elif 149 <= int(distance) <= 151:
            aspect = "Quincunx"
            return True, aspect, distance - 150
        elif 71.5 <= int(distance) <= 72.5:
            aspect = "Quintile"
            return True, aspect, distance - 72
        elif 143.5 <= int(distance) <= 144.5:
            aspect = "BiQuintile"
            return True, aspect, distance - 144
        elif (self.first_points_list[pl_us1]["name"] == "Sun" and
              self.second_points_list[pl_us2]["name"] == "Sun" and
              self.first_points_list[pl_us1]["quality"] ==
              self.second_points_list[pl_us2]["quality"]):
            aspect = "DESTINO"
            return False, aspect, distance
        else:
            return False, None, None

    def get_synastry(self):
        """
        Asingns points to the aspects and prints them.
        """
        lenght = len(self.first_points_list)
        counter = 0
        points = 0
        aspects = []
        value = 0
        for i in range(lenght):
            for b in range(lenght):
                verdict, aspect, orbit = self.synastry_aspect(self.user_a, i,
                                                              self.user_b, b)
                if verdict == True:
                    if aspect == "Conjuction" and ((self.first_points_list[i]
                                                    ["name"]
                                                    == "Sun" and self.second_points_list[b]["name"]
                                                    == "Moon") or (self.first_points_list[i]["name"]
                                                                   == "Moon" and self.second_points_list[b]["name"]
                                                                   == "Sun")):
                        value = 8
                        if abs(int(orbit)) <= 2:
                            value += 3
                            points += value
                        else:
                            points += value

                    elif (self.first_points_list[b]["name"]
                          == "Sun" and self.second_points_list[i]["name"]
                          == "Sun") or (self.first_points_list[b]["name"]
                                        == "Sun" and self.second_points_list[i]["name"]
                                        == "Moon") or (self.first_points_list[b]["name"]
                                                       == "Moon" and self.second_points_list[i]["name"]
                                                       == "Sun") or (self.first_points_list[b]["name"]
                                                                     == "Moon" and self.second_points_list[i]["name"]
                                                                     == "Moon") or (self.first_points_list[b]["name"]
                                                                                    == "1" and self.second_points_list[i]["name"]
                                                                                    == "1") or (self.first_points_list[b]["name"]
                                                                                                == "Sun" and self.second_points_list[i]["name"]
                                                                                                == "1") or (self.first_points_list[b]["name"]
                                                                                                            == "1" and self.second_points_list[i]["name"]
                                                                                                            == "Sun") or (self.first_points_list[b]["name"]
                                                                                                                          == "Moon" and self.second_points_list[i]["name"]
                                                                                                                          == "1") or (self.first_points_list[b]["name"]
                                                                                                                                      == "1" and self.second_points_list[i]["name"]
                                                                                                                                      == "Moon"):
                        value = 4
                        if (self.first_points_list[i]["name"]
                            == "Sun" and self.second_points_list[b]["name"]
                            == "Sun" and self.first_points_list[i]["sign"]
                                == self.second_points_list[b]["sign"]):
                            value += 5
                            points += value
                        else:
                            points += value

                    elif (self.first_points_list[i]["name"]
                          == "Venus" and self.second_points_list[b]["name"]
                          == "Mars") or (self.first_points_list[i]["name"]
                                         == "Mars" and self.second_points_list[b]["name"]
                                         == "Venus"):
                        value = 4
                        points += value
                    else:
                        value = 0

                    counter += 1
                    orbit = round(orbit, ORBIT_ROUND)
                    aspect = {"user_a_name":     self.user_a.name,
                              "user_a_object":   self.first_points_list[i]["name"],
                              "aspect":         aspect,
                              "user_b_name":     self.user_b.name,
                              "user_b_objcet":   self.second_points_list[b]["name"],
                              "orbit":          orbit,
                              "Value":          value,
                              "counter":        counter
                              }
                    aspects.append(aspect)

                elif verdict == False and aspect == "DESTINO":

                    counter += 1
                    value = 5
                    points += value
                    orbit = round(orbit, ORBIT_ROUND)

                    aspect = {'user_a_name':    self.user_a.name,
                              'user_a_object':  'Sun',
                              'aspect':        'DESTINY',
                              'user_b_name':    self.user_b.name,
                              'user_b_objcet':  'Sun',
                              'orbit':         orbit,
                              'Value':         value,
                              'counter':       counter
                              }

                    aspects.append(aspect)
                elif verdict == False:
                    # aspect = (self.user_a.name, self.first_points_list[i]["name"], aspect,
                    # self.user_b.name, self.second_points_list[b]["name"], orbit, counter)
                    None
                else:
                    aspect = ("ERROR")
                    aspects.append(aspect)

        self.counter = counter
        self.score = points
        self.aspects = aspects


if __name__ == "__main__":

    #lui = KrInstance("Windsor", 1894, 6, 23, 21, 55, "London")
    lui = KrInstance("Kanye", 1977, 6, 8, 8, 45,
                     "Atlanta", lon=50, lat=50, tz_str="Europe/Rome")

    #lei = KrInstance("Simpson", 1896, 6, 19, 22, 30, "Blue Ridge Summit")
    lei = KrInstance("Kanye", 1977, 6, 8, 8, 45,
                     "Atlanta", lon=50, lat=50, tz_str="Europe/Rome")

    sn = DiscepoloNumber(lei, lui)
    sn.get_synastry()
    print(lei.sun)
    print(lui.sun)

    for a in sn.aspects:
        print(a)

    print(sn.score)



from kerykeion.utilities import kr_settings
from os.path import abspath

def print_settings_path():
    path = abspath(kr_settings.__file__)
    print(path)


if __name__ == "__main__":
    print_settings_path()


from kerykeion.main import KrInstance
from kerykeion.utilities.kr_settings import aspects, colors, planets, axes_orbit
from swisseph import difdeg2n





class NatalAspects():

    """
    Generates an object with all the aspects of a birthcart.
    """
    
    def __init__(self, kr_object):
        self.user = kr_object
        
        if not hasattr(self.user, "sun"):
            print(f"Generating kerykeion object for {self.user.name}...")
            self.user.get_all()
        
        self.init_point_list = self.user.planets_list + self.user.houses_list

    

    def asp_calc(self, point_one, point_two):
        """ 
        Utility function.
        It calculates the aspects between the 2 points.
        Args: first point, second point. 
        """

        distance = abs(difdeg2n(point_one, point_two))
        diff = abs(point_one - point_two)

        if int(distance) <= aspects[0]['orb']:
            name = aspects[0]['name']
            aspect_degrees = aspects[0]['degree']
            color = colors['aspect_0']
            verdict = True
            aid = 0

        elif ( aspects[1]['degree'] - aspects[1]['orb']) <= int(distance) <= ( aspects[1]['degree'] + aspects[1]['orb']):
            name = aspects[1]['name']
            aspect_degrees = aspects[1]['degree']
            color = colors['aspect_30']
            verdict = True
            aid = 1

        elif ( aspects[2]['degree'] - aspects[2]['orb']) <= int(distance) <= ( aspects[2]['degree'] + aspects[2]['orb']):
            name = aspects[2]['name']
            aspect_degrees = aspects[2]['degree']
            color = colors['aspect_45']
            verdict = True
            aid = 2
        
        elif ( aspects[3]['degree'] - aspects[3]['orb']) <= int(distance) <= ( aspects[3]['degree'] + aspects[3]['orb']):
            name = aspects[3]['name']
            aspect_degrees = aspects[3]['degree']
            color = colors['aspect_60']
            verdict = True
            aid = 3
        
        elif ( aspects[4]['degree'] - aspects[4]['orb']) <= int(distance) <= ( aspects[4]['degree'] + aspects[4]['orb']):
            name = aspects[4]['name']
            aspect_degrees = aspects[4]['degree']
            color = colors['aspect_72']
            verdict = True
            aid = 4
        
        elif ( aspects[5]['degree'] - aspects[5]['orb']) <= int(distance) <= ( aspects[5]['degree'] + aspects[5]['orb']):
            name = aspects[5]['name']
            aspect_degrees = aspects[5]['degree']
            color = colors['aspect_90']
            verdict = True
            aid = 5

        elif ( aspects[6]['degree'] - aspects[6]['orb']) <= int(distance) <= ( aspects[6]['degree'] + aspects[6]['orb']):
            name = aspects[6]['name']
            aspect_degrees = aspects[6]['degree']
            color = colors['aspect_120']
            verdict = True
            aid = 6


        elif ( aspects[7]['degree'] - aspects[7]['orb']) <= int(distance) <= ( aspects[7]['degree'] + aspects[7]['orb']):
            name = aspects[7]['name']
            aspect_degrees = aspects[7]['degree']
            color = colors['aspect_135']
            verdict = True
            aid = 7
        
        elif ( aspects[8]['degree'] - aspects[8]['orb']) <= int(distance) <= ( aspects[8]['degree'] + aspects[8]['orb']):
            name = aspects[8]['name']
            aspect_degrees = aspects[8]['degree']
            color = colors['aspect_144']
            verdict = True
            aid = 8
        
        elif ( aspects[9]['degree'] - aspects[9]['orb']) <= int(distance) <= ( aspects[9]['degree'] + aspects[9]['orb']):
            name = aspects[9]['name']
            aspect_degrees = aspects[9]['degree']
            color = colors['aspect_150']
            verdict = True
            aid = 9


        elif ( aspects[10]['degree'] - aspects[10]['orb']) <= int(distance) <= ( aspects[10]['degree'] + aspects[10]['orb']):
            name = aspects[10]['name']
            aspect_degrees = aspects[10]['degree']
            color = colors['aspect_180']
            verdict = True
            aid = 10

        else:
            verdict = False
            name = None
            distance = 0
            aspect_degrees = 0
            color = None
            aid = None
            
        
        return verdict, name, distance - aspect_degrees, aspect_degrees, color, aid, diff
    
    def p_id_decoder(self, name):

        """ 
        Check if the name of the planet is the same in the settings and return
        the correct id for the planet.
        """
        str_name = str(name)
        for a in planets:
            if a['name'] == str_name:
                result = a['id']
                return result

    def filter_by_settings(self, init_point_list):
        """
        Creates a list of all the desired
        points filtering by the settings.
        """

        set_points_name = []
        for p in planets:
            if p['visible']:
                set_points_name.append(p['name'])
        
        point_list = []
        for l in init_point_list:
            if l['name'] in set_points_name:
                point_list.append(l)
        
        return point_list 
        

    def get_all_aspects(self):
        """
        Return all the aspects of the points in the natal chart in a dictionary,
        first all the individual aspects of each planet, second the aspects
        whitout repetitions.
        """

        point_list = self.filter_by_settings(self.init_point_list)
        
        self.all_aspects_list = []

        for first in range(len(point_list)):
        #Generates the aspects list whitout repetitions
            for second in range(first + 1, len(point_list)):
                
                verdict, name, orbit, aspect_degrees, color, aid, diff = self.asp_calc(point_list[first]["abs_pos"],
                point_list[second]["abs_pos"])
                
                if verdict == True:
                    d_asp = { "p1_name": point_list[first]['name'],
                              "p1_abs_pos": point_list[first]['abs_pos'],
                              "p2_name": point_list[second]['name'],
                              "p2_abs_pos": point_list[second]['abs_pos'],
                              "aspect": name,
                              "orbit": orbit,
                              "aspect_degrees": aspect_degrees,
                              "color": color,
                              "aid": aid,
                              "diff": diff,
                              "p1": self.p_id_decoder(point_list[first]['name']),
                              "p2": self.p_id_decoder(point_list[second]['name'],)
                     }

                    self.all_aspects_list.append(d_asp)

        return self.all_aspects_list


   

    def get_aspects(self):

        """ 
        Filters the aspects list with the desired points, in this case
        the most important are hardcoded.
        Set the list with set_points and creating a list with the names
        or the numbers of the houses.
        """
            
        self.get_all_aspects()

        aspects_filtered = []
        for a in self.all_aspects_list:
            if aspects[a["aid"]]["visible"] == True:
                aspects_filtered.append(a)  



        axes_list = ["1", "10", "7", "4"]
        counter = 0

        aspects_list_subtract = []
        for a in aspects_filtered:
            counter += 1
            name_p1 = str(a['p1_name'])
            name_p2 = str(a['p2_name'])
            
            if name_p1 in axes_list:
                if abs(a['orbit']) >= axes_orbit:
                    aspects_list_subtract.append(a)

            elif name_p2 in axes_list:
                if abs(a['orbit']) >= axes_orbit:
                    aspects_list_subtract.append(a)


        self.aspects = [item for item in aspects_filtered if item not in aspects_list_subtract]


        return self.aspects

class CompositeAspects(NatalAspects):
    """
    Generates an object with all the aspects between two persons.
    """

    def __init__(self, kr_object_one, kr_object_two):

        self.first_user = kr_object_one
        self.second_user = kr_object_two

        
        if not hasattr(self.first_user, "sun"):
            print(f"Generating kerykeion object for {self.first_user.name}...")
            self.first_user.get_all()
        
        if not hasattr(self.second_user, "sun"):
            print(f"Generating kerykeion object for {self.second_user.name}...")
            self.second_user.get_all()
        

        
        self.first_init_point_list = self.first_user.planets_list + self.first_user.houses_list
        self.second_init_point_list = self.second_user.planets_list + self.second_user.houses_list
        

    
    
    def get_all_aspects(self):
        """
        Return all the aspects of the points in the natal chart in a dictionary,
        first all the individual aspects of each planet, second the aspects
        whitout repetitions.
        """

        f_1 = self.filter_by_settings(self.first_init_point_list)
        f_2 = self.filter_by_settings(self.second_init_point_list)
        
        self.all_aspects_list = []

        for first in range(len(f_1)):
        #Generates the aspects list whitout repetitions
            for second in range(len(f_2)):
                
                verdict, name, orbit, aspect_degrees, color, aid, diff = self.asp_calc(f_1[first]["abs_pos"],
                f_2[second]["abs_pos"])
                
                if verdict == True:
                    d_asp = { "p1_name": f_1[first]['name'],
                              "p1_abs_pos": f_1[first]['abs_pos'],
                              "p2_name": f_2[second]['name'],
                              "p2_abs_pos": f_2[second]['abs_pos'],
                              "aspect": name,
                              "orbit": orbit,
                              "aspect_degrees": aspect_degrees,
                              "color": color,
                              "aid": aid,
                              "diff": diff,
                              "p1": self.p_id_decoder(f_1[first]['name']),
                              "p2": self.p_id_decoder(f_2[second]['name'],)
                     }

                    self.all_aspects_list.append(d_asp)

        return self.all_aspects_list
    

if __name__ == "__main__":
    kanye = KrInstance("Kanye", 1977, 6, 8, 8, 45, "Atlanta")
    jack = KrInstance("Jack", 1990, 6, 15, 13, 00, "Montichiari")
    # kanye.get_all()
    # natal = NatalAspects(kanye)
    # natal.get_aspects()
    # for a in natal.aspects:
    #     print(a['p1_name'], a['p2_name'], a['orbit'])
    cm = CompositeAspects(kanye, jack)
    res = cm.get_aspects()
    for a in res:
        print(a['p1_name'], a['p2_name'], a['orbit'], a['aspect'])
    print(len(res))

    
    
    
        



# -*- coding: utf-8 -*-
"""
    This file is part of openastro.org.

    OpenAstro.org is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenAstro.org is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenAstro.org.  If not, see <https://www.gnu.org/licenses/>.
"""

"""
docs at:
    https://www.geonames.org/export/geonames-search.html

preset vars:
    featureClass=P (populated place)
    maxRows=1
"""

from urllib.request import urlopen
from urllib.parse import urlencode
from xml.dom.minidom import parseString
from socket import timeout
from urllib.error import HTTPError, URLError
from os import mkdir, path, getcwd, remove
import json
import logging


logging.basicConfig(
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=logging.INFO)


def _getText(nodelist):
    """"
    Internal function to return text from nodes
    """
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc


def geonames_proxy(operation: str, params: str):
    # directory to store cache files in
    cache_directory = '.geonames_cache'

    # supported API calls and their corresponding file cache names
    cache_file_name = {
        'search': 'geonames_search_cache.json',
        'timezone': 'geonames_timezone_cache.json'
    }
    # supported API calls and their corresponding API endpoints
    api_endpoints = {
        'search': "http://api.geonames.org/search?%s",
        'timezone': "http://api.geonames.org/timezone?%s"
    }
    cache_file_name = cache_file_name[operation]
    cache_file_path = path.join(cache_directory, cache_file_name)
    api_endpoint = api_endpoints[operation]

    try:
        # try to create a directory to store the cache in
        # might fail under wrong permission settings
        if not path.exists(cache_directory):
            # mkdir if it doesn't already exist
            mkdir(cache_directory)
    except:
        logging.error(f'Failed to create directory: {cache_directory}')
        logging.error(f'Maybe check permissions in: "{getcwd()}"?')
        raise Exception(f'Error: failed to create {cache_directory} directory.')

    # dict to store the cache data in the program
    cache_dict = dict()
    try:
        # check if file exists
        if path.exists(cache_file_path) and path.getsize(cache_file_path) > 0:
            # then open it for reading with ability to write
            # an empty file is definitely not useful to json.load() so we'll just skip that and create a new one
            cache_file = open(cache_file_path, 'r+', encoding='utf-8')
            # then load data from it
            try:
                # this can fail if json file is corrupted
                cache_dict = json.load(cache_file)
                logging.debug(f'Loaded proxy cache file: "{cache_file_path}" and parsed as JSON.')

            except:
                logging.error(f'Failed to parse JSON file: "{cache_file_path}"!')
                cache_file.seek(0)
                logging.error(f'Contents: {cache_file.read()}')
                logging.error(f'Try removing: {path.join(getcwd(),cache_directory)}')
                raise Exception('Error: Failed to parse JSON file!')
        else:
            # if file doesn't exist or is empty create it for writing with ability to read
            cache_file = open(cache_file_path, 'w+', encoding='utf-8')

    except:
        logging.debug(f'Failed to access: {cache_file_path}')
        raise Exception('Error: failed to access Geonames cache file.')

    if params not in cache_dict or cache_dict[params] == "":
        # if the file does not contain the key with the parameters proceed into the branch
        # if the file does contain the key with the param but it's empty then we'll replace it
        # if the file does already contain the API response then this branch is skipped and data from the file is
        # returned instead
        if params in cache_dict and cache_dict[params] == "":
            logging.warning(f'Proxy cache file has an empty value for key:"{params}".')
        elif params not in cache_dict:
            logging.debug(f'Params: "{params}" not found in proxy cache file.')
        try:
            # try to query geonames
            logging.debug(f'Calling Geonames API endpoint: "{api_endpoint}" with params: "{params}".')
            http_response = urlopen(api_endpoint % params, timeout=20)

        except (HTTPError, URLError) as error:
            logging.error(f'Not retrieved because {error}\nURL: {cache_dict[params]}')
            raise Exception('Error: failed to access Geonames service.')

        except timeout:
            logging.error('Timeout on search in Geonames API!')
            raise Exception('Error: Timeout on connection with Geonames.')

        # store the response in the dict
        data = http_response.read()
        cache_dict[params] = data.decode('UTF-8')

        # empty the file. Without this json.dump() will append to the existing data
        cache_file.close()
        cache_file = open(cache_file_path, 'w', encoding='utf-8')

        # dump the dict in JSON format into the file
        json.dump(cache_dict, cache_file)
        cache_file.close()
        logging.debug('Saved new proxy cache file.')

    return cache_dict[params], cache_file_path


def search(name='', country=''):
    """Search function for geonames.org api
            name must be supplied
            country is optional, 2 character country code
    """
    # check name
    if name == '':
        logging.error('No name specified for Geonames search!')
        return None

    # open connection and read xml
    params = urlencode({'q': name, 'country': country, 'maxRows': 1,
                        'featureClass': 'P', 'username': 'century.boy'})

    try:
        data, cache_file_path = geonames_proxy('search', params)
        dom = parseString(data)
    except Exception:
        return None

    # totalResultsCount
    try:
        totalResultsCount = _getText(
            dom.getElementsByTagName("totalResultsCount")[0].childNodes)
    except:
        logging.error("Geonames API response could not be parsed! Maybe you exceeded API hourly limit?")
        logging.error(f"String that failed to parse: {data}")
        # remove the cache_file since it now contains the hourly limit error message
        remove(cache_file_path)
        return None


    # geoname
    geoname = []
    for i in dom.getElementsByTagName("geoname"):
        geoname.append({})
        geoname[-1]['name'] = _getText(
            i.getElementsByTagName("name")[0].childNodes)
        geoname[-1]['lat'] = _getText(i.getElementsByTagName("lat")
                                      [0].childNodes)
        geoname[-1]['lng'] = _getText(i.getElementsByTagName("lng")
                                      [0].childNodes)
        geoname[-1]['geonameId'] = _getText(
            i.getElementsByTagName("geonameId")[0].childNodes)
        geoname[-1]['countryCode'] = _getText(
            i.getElementsByTagName("countryCode")[0].childNodes)
        geoname[-1]['countryName'] = _getText(
            i.getElementsByTagName("countryName")[0].childNodes)
        geoname[-1]['fcl'] = _getText(i.getElementsByTagName("fcl")
                                      [0].childNodes)
        geoname[-1]['fcode'] = _getText(
            i.getElementsByTagName("fcode")[0].childNodes)
        # get timezone
        tparams = urlencode(
            {'lat': geoname[-1]['lat'], 'lng': geoname[-1]['lng'], 'username': 'century.boy'})
        try:
            data, cache_file_path = geonames_proxy('timezone', tparams)
        except Exception:
            return None

        tdom = parseString(data)

        try:
            geoname[-1]['timezonestr'] = _getText(tdom.getElementsByTagName("timezoneId")[0].childNodes)
        except:
            logging.error("Geonames API response could not be parsed! Maybe you exceeded API hourly limit?")
            logging.error(f"String that failed to parse: {data}")
            # remove the cache_file since it now contains the hourly limit error message
            remove(cache_file_path)
            return None

        tdom.unlink()
        break
    # close dom
    dom.unlink()

    # return results
    if totalResultsCount == "0":
        logging.error("No results found using Geonames API!")
        return None
    else:
        return geoname



"""
    This is part of Kerykeion (C) 2020 Giacomo Battaglia
"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from kerykeion.main import KrInstance

def output(user):
    p = user.planets_in_houses()
    h = user.houses()
    finale = ("-----------------------------------------------------\n")
    finale += ("NAME: " + user.name + "\n")
    finale += ("PLANET     POSITION\n")
    finale += ("                      \n") 
    finale += (f"Sun:       {user.sun['sign']} {round(user.sun['position'], 3)} in {user.sun['house']}\n")
    finale += (f"Moon:      {user.moon['sign']} {round(user.moon['position'], 3)} in {user.moon['house']}\n")
    finale += (f"Mercury:   {user.mercury['sign']} {round(user.mercury['position'], 3)} in {user.mercury['house']}\n")
    finale += (f"Venus:     {user.venus['sign']} {round(user.venus['position'], 3)} in {user.venus['house']}\n")
    finale += (f"Mars:      {user.mars['sign']} {round(user.mars['position'], 3)} in {user.mars['house']}\n")
    finale += (f"Jupiter:   {user.jupiter['sign']} {round(user.jupiter['position'], 3)} in {user.jupiter['house']}\n")
    finale += (f"Saturn:    {user.saturn['sign']} {round(user.saturn['position'], 3)} in {user.saturn['house']}\n")
    finale += (f"Uranus:    {user.uranus['sign']} {round(user.uranus['position'], 3)} in {user.uranus['house']}\n")
    finale += (f"Neptune:   {user.neptune['sign']} {round(user.neptune['position'], 3)} in {user.neptune['house']}\n")
    finale += (f"Pluto:     {user.pluto['sign']} {round(user.pluto['position'], 3)} in {user.pluto['house']}\n")
    #finale += (f"Juno:      {p[10]['sign']} {round(p[10]['pos'], 3)} in {p[10]['house']}\n\n")
    finale += ("\nPLACIDUS HOUSES\n")
    finale += (f"House Cusp 1:     {user.first_house['sign']}  {round(user.first_house['position'], 3)}\n")
    finale += (f"House Cusp 2:     {user.second_house['sign']}  {round(user.second_house['position'], 3)}\n")
    finale += (f"House Cusp 3:     {user.third_house['sign']}  {round(user.third_house['position'], 3)}\n")
    finale += (f"House Cusp 4:     {user.fourth_house['sign']}  {round(user.fourth_house['position'], 3)}\n")
    finale += (f"House Cusp 5:     {user.fifth_house['sign']}  {round(user.fifth_house['position'], 3)}\n")
    finale += (f"House Cusp 6:     {user.sixth_house['sign']}  {round(user.sixth_house['position'], 3)}\n")
    finale += (f"House Cusp 7:     {user.seventh_house['sign']}  {round(user.seventh_house['position'], 3)}\n")
    finale += (f"House Cusp 8:     {user.eighth_house['sign']}  {round(user.eighth_house['position'], 3)}\n")
    finale += (f"House Cusp 9:     {user.ninth_house['sign']}  {round(user.ninth_house['position'], 3)}\n")
    finale += (f"House Cusp 10:    {user.tenth_house['sign']}  {round(user.tenth_house['position'], 3)}\n")
    finale += (f"House Cusp 11:    {user.eleventh_house['sign']}  {round(user.eleventh_house['position'], 3)}\n")
    finale += (f"House Cusp 12:    {user.twelfth_house['sign']}  {round(user.twelfth_house['position'], 3)}\n")
    finale += ("\n")
    return finale





if __name__ == "__main__":
    user = KrInstance("Brigitte Bardot", 1934, 9, 28, 13, 15, "Paris", "FR")
    print(output(user))



import sys
from os.path import dirname
from kerykeion.main import KrInstance
from kerykeion.output_exemple import output
from kerykeion.utilities import kr_settings as settings
from kerykeion.utilities.general import print_settings_path
from kerykeion.utilities.charts import MakeSvgInstance as MakeSvgInstance
from kerykeion.utilities.discepolo import DiscepoloNumber


sys.path.append(dirname(__file__))



"""
    This is part of Kerykeion (C) 2020 Giacomo Battaglia
"""

import datetime
from kerykeion.geoname import search
import logging
import math
import os.path
import pytz
from sys import exit
import swisseph as swe
from typing import Union

# swe.set_ephe_path("/")

DEBUG = False

logging.basicConfig(
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=logging.INFO)


def debug_print(str):
    if DEBUG:
        print('%s' % str)


class KrInstance():
    """
    Calculates all the astrological informations, the coordinates,
    it's utc and julian day and returns an object with all that data.
    Args: name, year, month, day, hours, minuts, city,
    initial of the nation (Ex: "IT"),
    longitude, latitude and the timezone string are set to false so they will
    be calulated, if you want you can set them.
    Default values are set for now at greenwich time.

    """
    now = datetime.datetime.now()

    def __init__(
        self,
        name: str = "Now",
        year: int = now.year,
        month: int = now.month,
        day: int = now.day,
        hours: int = now.hour,
        minuts: int = now.minute,
        city: str = "London",
        nat: str = "",
        lon: Union[int, float] = False,
        lat: Union[int, float] = False,
        tz_str: Union[str, bool] = False
    ):
        logging.info('Starting Kerykeion')
        self.name = name
        self.year = year
        self.month = month
        self.day = day
        self.hours = hours
        self.minuts = minuts
        self.city = city
        self.nation = nat
        self.city_long = lon
        self.city_lat = lat
        self.city_tz = tz_str

        self.json_dir = os.path.expanduser("~")

        self.julian_day = self.get_jd()
        self.zodiactype = "Tropic"

        # Get all the calculations
        self.get_all()

    def __str__(self):
        return f"Astrological data for: {self.name}, {self.utc} UTC"

    def __repr__(self) -> str:
        return f"Astrological data for: {self.name}, {self.utc} UTC"

    def get_tz(self):
        """Gets the nerest time zone for the calculation"""
        logging.debug("Conneting to Geonames...")
        try:
            self.city_data = search(self.city, self.nation)[0]
        except:
            exit('Error connecting to geonames, try again!')

        logging.debug("Geonames done!")

        self.nation = self.city_data["countryCode"]
        self.city_long = float(self.city_data["lng"])
        self.city_lat = float(self.city_data["lat"])
        self.city_tz = self.city_data["timezonestr"]

        # self.country_code = self.city_data["countryCode"]

        if self.city_lat > 66.0:
            self.city_lat = 66.0
            print("polar circle override for houses, using 66 degrees")

        elif self.city_lat < -66.0:
            self.city_lat = -66.0
            print("polar circle override for houses, using -66 degrees")

        return self.city_tz

    def get_utc(self):
        """Converts local time to utc time. """

        if not self.city_long or not self.city_lat or not self.city_tz:
            local_time = pytz.timezone(self.get_tz())
        else:
            local_time = pytz.timezone(self.city_tz)

        naive_datetime = datetime.datetime(self.year, self.month,
                                           self.day, self.hours, self.minuts, 0)
        local_datetime = local_time.localize(naive_datetime, is_dst=None)
        utc_datetime = local_datetime.astimezone(pytz.utc)
        self.utc = utc_datetime
        return self.utc

    def get_jd(self):
        """ Calculates julian day from the utc time."""
        utc = self.get_utc()
        self.time_utc = utc.hour + utc.minute/60
        self.time = self.hours + self.minuts/60
        self.julian_day = float(swe.julday(utc.year, utc.month, utc.day,
                                           self.time_utc))

        return self.julian_day

    @staticmethod
    def get_number(name):
        """Internal function, gets number id from the name."""
        name = name.lower()

        if name == "sun":
            return 0
        elif name == "moon":
            return 1
        elif name == "mercury":
            return 2
        elif name == "venus":
            return 3
        elif name == "mars":
            return 4
        elif name == "jupiter":
            return 5
        elif name == "saturn":
            return 6
        elif name == "uranus":
            return 7
        elif name == "neptune":
            return 8
        elif name == "pluto":
            return 9
        elif name == "mean_node":
            return 10  # change!
        elif name == "true_node":
            return 11
        else:
            return int(name)

    @staticmethod
    def position_calc(degree, number_name, label):
        """A function to be used in others to create a dictionary deviding
        the houses or the planets list."""
        if degree < 30:
            dictionary = {label: number_name, "quality": "Cardinal", "element":
                          "Fire", "sign": "Ari", "sign_num": 0, "position": degree, "abs_pos": degree,
                          "emoji": "♈️"}
        elif degree < 60:
            result = degree - 30
            dictionary = {label: number_name, "quality": "Fixed", "element":
                          "Earth", "sign": "Tau", "sign_num": 1, "position": result, "abs_pos": degree,
                          "emoji": "♉️"}
        elif degree < 90:
            result = degree - 60
            dictionary = {label: number_name, "quality": "Mutable", "element":
                          "Air", "sign": "Gem", "sign_num": 2, "position": result, "abs_pos": degree,
                          "emoji": "♊️"}
        elif degree < 120:
            result = degree - 90
            dictionary = {label: number_name, "quality": "Cardinal", "element":
                          "Water", "sign": "Can", "sign_num": 3, "position": result, "abs_pos": degree,
                          "emoji": "♋️"}
        elif degree < 150:
            result = degree - 120
            dictionary = {label: number_name, "quality": "Fixed", "element":
                          "Fire", "sign": "Leo", "sign_num": 4, "position": result, "abs_pos": degree,
                          "emoji": "♌️"}
        elif degree < 180:
            result = degree - 150
            dictionary = {label: number_name, "quality": "Mutable", "element":
                          "Earth", "sign": "Vir", "sign_num": 5, "position": result, "abs_pos": degree,
                          "emoji": "♍️"}
        elif degree < 210:
            result = degree - 180
            dictionary = {label: number_name, "quality": "Cardinal", "element":
                          "Air", "sign": "Lib", "sign_num": 6, "position": result, "abs_pos": degree,
                          "emoji": "♎️"}
        elif degree < 240:
            result = degree - 210
            dictionary = {label: number_name, "quality": "Fixed", "element":
                          "Water", "sign": "Sco", "sign_num": 7, "position": result, "abs_pos": degree,
                          "emoji": "♏️"}
        elif degree < 270:
            result = degree - 240
            dictionary = {label: number_name, "quality": "Mutable", "element":
                          "Fire", "sign": "Sag", "sign_num": 8, "position": result, "abs_pos": degree,
                          "emoji": "♐️"}
        elif degree < 300:
            result = degree - 270
            dictionary = {label: number_name, "quality": "Cardinal", "element":
                          "Earth", "sign": "Cap", "sign_num": 9, "position": result, "abs_pos": degree,
                          "emoji": "♑️"}
        elif degree < 330:
            result = degree - 300
            dictionary = {label: number_name, "quality": "Fixed", "element":
                          "Air", "sign": "Aqu", "sign_num": 10, "position": result, "abs_pos": degree,
                          "emoji": "♒️"}
        elif degree < 360:
            result = degree - 330
            dictionary = {label: number_name, "quality": "Mutable", "element":
                          "Water", "sign": "Pis", "sign_num": 11, "position": result, "abs_pos": degree,
                          "emoji": "♓️"}
        else:
            dictionary = {label: "position_calc error", "sign": "position_calc error",
                          "position": "position_calc error"}

        return dictionary

    def houses(self):
        """Calculatetype positions and store them in dictionaries"""

        # creates the list of the house in 360°
        self.houses_degree_ut = swe.houses(self.julian_day, self.city_lat,
                                           self.city_long)[0]
        # stores the house in signulare dictionaries.
        self.first_house = self.position_calc(
            self.houses_degree_ut[0], "1", "name")
        self.second_house = self.position_calc(
            self.houses_degree_ut[1], "2", "name")
        self.third_house = self.position_calc(
            self.houses_degree_ut[2], "3", "name")
        self.fourth_house = self.position_calc(
            self.houses_degree_ut[3], "4", "name")
        self.fifth_house = self.position_calc(
            self.houses_degree_ut[4], "5", "name")
        self.sixth_house = self.position_calc(
            self.houses_degree_ut[5], "6", "name")
        self.seventh_house = self.position_calc(
            self.houses_degree_ut[6], "7", "name")
        self.eighth_house = self.position_calc(
            self.houses_degree_ut[7], "8", "name")
        self.ninth_house = self.position_calc(
            self.houses_degree_ut[8], "9", "name")
        self.tenth_house = self.position_calc(
            self.houses_degree_ut[9], "10", "name")
        self.eleventh_house = self.position_calc(
            self.houses_degree_ut[10], "11", "name")
        self.twelfth_house = self.position_calc(
            self.houses_degree_ut[11], "12", "name")

        # creates a list of all the dictionaries of thetype.

        houses_degree = [self.first_house["position"], self.second_house["position"],
                         self.third_house["position"], self.fourth_house["position"], self.fifth_house["position"],
                         self.sixth_house["position"], self.seventh_house, self.eighth_house["position"],
                         self.ninth_house["position"], self.tenth_house["position"], self.eleventh_house["position"],
                         self.twelfth_house["position"]]

        # return self.houses_list
        return houses_degree

    def planets_deegrees_lister(self):
        """Sidereal or tropic mode."""
        self.iflag = swe.FLG_SWIEPH+swe.FLG_SPEED

        if self.zodiactype == "sidereal":
            self.iflag += swe.FLG_SIDEREAL
            mode = "SIDM_FAGAN_BRADLEY"
            swe.set_sid_mode(getattr(swe, mode))

        """Calculates the position of the planets and stores it in a list."""

        sun_deg = swe.calc(self.julian_day, 0, self.iflag)[0][0]
        moon_deg = swe.calc(self.julian_day, 1, self.iflag)[0][0]
        mercury_deg = swe.calc(self.julian_day, 2, self.iflag)[0][0]
        venus_deg = swe.calc(self.julian_day, 3, self.iflag)[0][0]
        mars_deg = swe.calc(self.julian_day, 4, self.iflag)[0][0]
        jupiter_deg = swe.calc(self.julian_day, 5, self.iflag)[0][0]
        saturn_deg = swe.calc(self.julian_day, 6, self.iflag)[0][0]
        uranus_deg = swe.calc(self.julian_day, 7, self.iflag)[0][0]
        neptune_deg = swe.calc(self.julian_day, 8, self.iflag)[0][0]
        pluto_deg = swe.calc(self.julian_day, 9, self.iflag)[0][0]
        mean_node_deg = swe.calc(self.julian_day, 10, self.iflag)[0][0]
        true_node_deg = swe.calc(self.julian_day, 11, self.iflag)[0][0]

        # print(swe.calc(self.julian_day, 7, self.iflag)[3])

        self.planets_degrees = [sun_deg, moon_deg, mercury_deg,
                                venus_deg, mars_deg, jupiter_deg, saturn_deg,
                                uranus_deg, neptune_deg, pluto_deg, mean_node_deg,
                                true_node_deg]

        return self.planets_degrees

    def planets(self):
        """ Defines body positon in signs and informations and
         stores them in dictionaries"""
        self.planets_degrees = self.planets_deegrees_lister()
        # stores the planets in signulare dictionaries.
        self.sun = self.position_calc(self.planets_degrees[0], "Sun", "name")
        self.moon = self.position_calc(self.planets_degrees[1], "Moon", "name")
        self.mercury = self.position_calc(
            self.planets_degrees[2], "Mercury", "name")
        self.venus = self.position_calc(
            self.planets_degrees[3], "Venus", "name")
        self.mars = self.position_calc(self.planets_degrees[4], "Mars", "name")
        self.jupiter = self.position_calc(
            self.planets_degrees[5], "Jupiter", "name")
        self.saturn = self.position_calc(
            self.planets_degrees[6], "Saturn", "name")
        self.uranus = self.position_calc(
            self.planets_degrees[7], "Uranus", "name")
        self.neptune = self.position_calc(
            self.planets_degrees[8], "Neptune", "name")
        self.pluto = self.position_calc(
            self.planets_degrees[9], "Pluto", "name")
        self.mean_node = self.position_calc(
            self.planets_degrees[10], "Mean_Node", "name")
        self.true_node = self.position_calc(
            self.planets_degrees[11], "True_Node", "name")

    def planets_in_houses(self):
        """Calculates the house of the planet and updates
        the planets dictionary."""
        self.planets()
        self.houses()

        def for_every_planet(planet, planet_deg):
            """Function to do the calculation.
            Args: planet dictionary, planet degree"""

            def point_between(p1, p2, p3):
                """Finds if a point is between two other in a circle
                args: first point, second point, point in the middle"""
                p1_p2 = math.fmod(p2 - p1 + 360, 360)
                p1_p3 = math.fmod(p3 - p1 + 360, 360)
                if (p1_p2 <= 180) != (p1_p3 > p1_p2):
                    return True
                else:
                    return False

            if point_between(self.houses_degree_ut[0], self.houses_degree_ut[1],
                             planet_deg) == True:
                planet["house"] = "1st House"
            elif point_between(self.houses_degree_ut[1], self.houses_degree_ut[2],
                               planet_deg) == True:
                planet["house"] = "2nd House"
            elif point_between(self.houses_degree_ut[2], self.houses_degree_ut[3],
                               planet_deg) == True:
                planet["house"] = "3rd House"
            elif point_between(self.houses_degree_ut[3], self.houses_degree_ut[4],
                               planet_deg) == True:
                planet["house"] = "4th House"
            elif point_between(self.houses_degree_ut[4], self.houses_degree_ut[5],
                               planet_deg) == True:
                planet["house"] = "5th House"
            elif point_between(self.houses_degree_ut[5], self.houses_degree_ut[6],
                               planet_deg) == True:
                planet["house"] = "6th House"
            elif point_between(self.houses_degree_ut[6], self.houses_degree_ut[7],
                               planet_deg) == True:
                planet["house"] = "7th House"
            elif point_between(self.houses_degree_ut[7], self.houses_degree_ut[8],
                               planet_deg) == True:
                planet["house"] = "8th House"
            elif point_between(self.houses_degree_ut[8], self.houses_degree_ut[9],
                               planet_deg) == True:
                planet["house"] = "9th House"
            elif point_between(self.houses_degree_ut[9], self.houses_degree_ut[10],
                               planet_deg) == True:
                planet["house"] = "10th House"
            elif point_between(self.houses_degree_ut[10], self.houses_degree_ut[11],
                               planet_deg) == True:
                planet["house"] = "11th House"
            elif point_between(self.houses_degree_ut[11], self.houses_degree_ut[0],
                               planet_deg) == True:
                planet["house"] = "12th House"
            else:
                planet["house"] = "error!"

            return planet

        self.sun = for_every_planet(self.sun, self.planets_degrees[0])
        self.moon = for_every_planet(self.moon, self.planets_degrees[1])
        self.mercury = for_every_planet(
            self.mercury, self.planets_degrees[2])
        self.venus = for_every_planet(
            self.venus, self.planets_degrees[3])
        self.mars = for_every_planet(self.mars, self.planets_degrees[4])
        self.jupiter = for_every_planet(
            self.jupiter, self.planets_degrees[5])
        self.saturn = for_every_planet(
            self.saturn, self.planets_degrees[6])
        self.uranus = for_every_planet(
            self.uranus, self.planets_degrees[7])
        self.neptune = for_every_planet(
            self.neptune, self.planets_degrees[8])
        self.pluto = for_every_planet(
            self.pluto, self.planets_degrees[9])
        self.mean_node = for_every_planet(
            self.mean_node, self.planets_degrees[10])
        self.true_node = for_every_planet(
            self.true_node, self.planets_degrees[11])

        planets_list = [self.sun, self.moon, self.mercury, self.venus,
                        self.mars, self.jupiter, self.saturn, self.uranus, self.neptune,
                        self.pluto, self.mean_node, self.true_node]

        # Check in retrograde or not:

        planets_ret = []
        for p in planets_list:
            planet_number = self.get_number(p["name"])
            if swe.calc(self.julian_day, planet_number, self.iflag)[0][3] < 0:
                p.update({'retrograde': True})
            else:
                p.update({'retrograde': False})
            planets_ret.append(p)

    def lunar_phase_calc(self):
        """ Function to calculate the lunar phase"""

        # anti-clockwise degrees between sun and moon
        moon, sun = self.planets_degrees[1], self.planets_degrees[0]
        degrees_between = moon - sun

        if degrees_between < 0:
            degrees_between += 360.0

        step = 360.0 / 28.0

        for x in range(28):
            low = x * step
            high = (x + 1) * step
            if degrees_between >= low and degrees_between < high:
                mphase = x + 1

        sunstep = [0, 30, 40,  50, 60, 70, 80, 90, 120, 130, 140, 150, 160, 170, 180,
                   210, 220, 230, 240, 250, 260, 270, 300, 310, 320, 330, 340, 350]

        for x in range(len(sunstep)):

            low = sunstep[x]

            if x == 27:
                high = 360
            else:
                high = sunstep[x+1]
            if degrees_between >= low and degrees_between < high:
                sphase = x + 1

        def moon_emoji(phase):
            if phase == 1:
                result = "🌑"
            elif phase < 7:
                result = "🌒"
            elif 7 <= phase <= 9:
                result = "🌓"
            elif phase < 14:
                result = "🌔"
            elif phase == 14:
                result = "🌕"
            elif phase < 20:
                result = "🌖"
            elif 20 <= phase <= 22:
                result = "🌗"
            elif phase <= 28:
                result = "🌘"
            else:
                result = phase

            return result

        self.lunar_phase = {
            "degrees_between_s_m": degrees_between,
            "moon_phase": mphase,
            "sun_phase": sphase,
            "moon_emoji": moon_emoji(mphase)
        }

    def make_lists(self):
        """ Internal function to generate the lists"""
        self.planets_list = [self.sun, self.moon, self.mercury, self.venus,
                             self.mars, self.jupiter, self.saturn, self.uranus, self.neptune,
                             self.pluto, self.mean_node, self.true_node]

        self.houses_list = [self.first_house, self.second_house, self.third_house,
                            self.fourth_house, self.fifth_house, self.sixth_house, self.seventh_house,
                            self.eighth_house, self.ninth_house, self.tenth_house, self.eleventh_house,
                            self.twelfth_house]

    def get_all(self):
        """ Gets all data from all the functions """

        self.planets_in_houses()
        # self.retrograde()
        self.lunar_phase_calc()
        self.make_lists()

    # Utility Functions:

    def json_dump(self, dump=True, new_output_directory=None):
        import jsonpickle, json

        """
        Dumps the Kerykeion object to a json file located in the home folder.
        This json file allows the object to be recreated with jsonpickle
        """

        try:
            self.sun
        except:
            self.get_all()

        if new_output_directory:
            self.json_path = os.path.join(
                new_output_directory, f"{self.name}_kerykeion.json")
        else:
            self.json_path = os.path.join(
                self.json_dir, f"{self.name}_kerykeion.json")

        json_string = jsonpickle.encode(self)
        
        hiden_values = [f' "json_dir": "{self.json_dir}",', f', "json_path": "{self.json_path}"'
                        ]

        for string in hiden_values:
            json_string = json_string.replace(string, "")

        if dump:
            json_string = json.loads(json_string.replace("'",'"'))
            with open(self.json_path, "w") as file:
                json.dump(json_string, file,  indent=4,sort_keys=True)
                print(f"JSON file dumped in {self.json_path}.")
        else:
            pass
        return json_string

    def json_api(self, dump=False):
        from json import dumps

        if not self.sun:
            self.get_all()

        obj_dict = self.__dict__.copy()
        keys_to_remove = ["utc", "iflag", "json_dir", "planets_list",
                          "houses_list", "planets_degrees", "houses_degree_ut"]
        for key in keys_to_remove:
            obj_dict.pop(key)

        json_obj = dumps(obj_dict)

        if dump:
            json_path = os.path.join(
                self.json_dir, f"{self.name}_kerykeion.json")
            with open(json_path, "w") as file:
                file.write(json_obj)
                print(f"JSON file dumped in {json_path}.")

        return json_obj


if __name__ == "__main__":
    # kanye = KrInstance("Kanye", 1977, 6, 8, 8, 45, "Atlanta",
    #                   lon = 50, lat = 50, tz_str = "Europe/Rome")
    # kanye = KrInstance("Kanye", 1977, 6, 8, 8, 45, "Atlanta")
    # kanye.get_all()
    # print(kanye.sun)
    # print(kanye.city_tz)
    # print(kanye.fifth_house["position"])
    # print(KrInstance().city)
    # print(KrInstance())

    #############################

    # f = kanye.json_dump(dump=True)
    # api_json = kanye.json_api()
    # print(api_json)

    # print(kanye.city)
    # print(f)
    # print(kanye.lunar_phase)
    # kanye.json_dump()

    # for p in kanye.planets_list:
    #     print(p)
    ###############################
    now = KrInstance("Kanye", 1977, 6, 8, 8, 45, "Atlanta",
        lon=50, lat=50, tz_str="Europe/Rome")

    kanye = KrInstance("Kanye", 1977, 6, 8, 8, 45, "Atlanta")               
    print(kanye.sun)



# Label settings

english = {
    "cusp": "Cusp",
    "longitude": "Longitude",
    "latitude": "Latitude",
    "north": "North",
    "east": "East",
    "south": "South",
    "west": "West",
    "fire": "Fire",
    "earth": "Earth",
    "air": "Air",
    "water": "Water",
    "&": "and",
    "transits": "Transits for",
    "type": "Type",
    "aspects": "Couple aspects",
    "planets_and_house": "Planets and houses for",
    "transit_name": "Here and now"

}


# Color settings:

colors = {
    "paper_0": "#000000",
    "paper_1": "#ffffff",
    "zodiac_bg_0": "#ff7200",
    "zodiac_bg_1": "#6b3d00",
    "zodiac_bg_2": "#69acf1",
    "zodiac_bg_3": "#2b4972",
    "zodiac_bg_4": "#ff7200",
    "zodiac_bg_5": "#6b3d00",
    "zodiac_bg_6": "#69acf1",
    "zodiac_bg_7": "#2b4972",
    "zodiac_bg_8": "#ff7200",
    "zodiac_bg_9": "#6b3d00",
    "zodiac_bg_10": "#69acf1",
    "zodiac_bg_11": "#2b4972",
    "zodiac_icon_0": "#ff7200",
    "zodiac_icon_1": "#6b3d00",
    "zodiac_icon_2": "#69acf1",
    "zodiac_icon_3": "#2b4972",
    "zodiac_icon_4": "#ff7200",
    "zodiac_icon_5": "#6b3d00",
    "zodiac_icon_6": "#69acf1",
    "zodiac_icon_7": "#2b4972",
    "zodiac_icon_8": "#ff7200",
    "zodiac_icon_9": "#6b3d00",
    "zodiac_icon_10": "#69acf1",
    "zodiac_icon_11": "#2b4972",
    "zodiac_radix_ring_0": "#ff0000",
    "zodiac_radix_ring_1": "#ff0000",
    "zodiac_radix_ring_2": "#ff0000",
    "zodiac_transit_ring_0": "#ff0000",
    "zodiac_transit_ring_1": "#ff0000",
    "zodiac_transit_ring_2": "#0000ff",
    "zodiac_transit_ring_3": "#0000ff",
    "houses_radix_line": "#ff0000",
    "houses_transit_line": "#0000ff",
    "aspect_0": "#5757e2",
    "aspect_30": "#810757",
    "aspect_45": "#b14e58",
    "aspect_60": "#d59e28",
    "aspect_72": "#1f99b3",
    "aspect_90": "#dc0000",
    "aspect_120": "#36d100",
    "aspect_135": "#985a10",
    "aspect_144": "#7a9810",
    "aspect_150": "#26bbcf",
    "aspect_180": "#510060",
    "planet_0": "#984b00",
    "planet_1": "#150052",
    "planet_2": "#520800",
    "planet_3": "#400052",
    "planet_4": "#540000",
    "planet_5": "#47133d",
    "planet_6": "#124500",
    "planet_7": "#6f0766",
    "planet_8": "#06537f",
    "planet_9": "#713f04",
    "planet_10": "#4c1541",
    "planet_11": "#4c1541",
    "planet_12": "#ff7e00",
    "planet_13": "#FF0000",
    "planet_14": "#0000FF",
    "planet_15": "#000000",
    "lunar_phase_0": "#000000",
    "lunar_phase_1": "#FFFFFF",
    "lunar_phase_2": "#CCCCCC"
}


# Aspects settings

aspects = [
    {
        "degree": 0,
        "name": "conjunction",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 1,
        "is_minor": 0,
        "orb": 10
    },
    {
        "degree": 30,
        "name": "semi-sextile",
        "visible": 0,
        "visible_grid": 0,
        "is_major": 0,
        "is_minor": 1,
        "orb": 1
    },
    {
        "degree": 45,
        "name": "semi-square",
        "visible": 0,
        "visible_grid": 0,
        "is_major": 0,
        "is_minor": 1,
        "orb": 1
    },
    {
        "degree": 60,
        "name": "sextile",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 1,
        "is_minor": 0,
        "orb": 6
    },
    {
        "degree": 72,
        "name": "quintile",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 0,
        "is_minor": 1,
        "orb": 1
    },
    {
        "degree": 90,
        "name": "square",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 1,
        "is_minor": 0,
        "orb": 5
    },
    {
        "degree": 120,
        "name": "trine",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 1,
        "is_minor": 0,
        "orb": 8
    },
    {
        "degree": 135,
        "name": "sesquiquadrate",
        "visible": 0,
        "visible_grid": 0,
        "is_major": 0,
        "is_minor": 1,
        "orb": 1
    },
    {
        "degree": 144,
        "name": "biquintile",
        "visible": 0,
        "visible_grid": 0,
        "is_major": 0,
        "is_minor": 1,
        "orb": 1
    },
    {
        "degree": 150,
        "name": "quincunx",
        "visible": 0,
        "visible_grid": 0,
        "is_major": 0,
        "is_minor": 0,
        "orb": 1
    },
    {
        "degree": 180,
        "name": "opposition",
        "visible": 1,
        "visible_grid": 1,
        "is_major": 1,
        "is_minor": 0,
        "orb": 10
    }
]

# Planets settings

planets = [
    {
        "id": 0,
        "name": "Sun",
        "color": "#984b00",
        "visible": 1,
        "element_points": 40,
        "zodiac_relation": "4",
        "label": "Sun",
        "label_short": "sun"
    },
    {
        "id": 1,
        "name": "Moon",
        "color": "#150052",
        "visible": 1,
        "element_points": 40,
        "zodiac_relation": "3",
        "label": "Moon",
        "label_short": "moon"
    },
    {
        "id": 2,
        "name": "Mercury",
        "color": "#520800",
        "visible": 1,
        "element_points": 15,
        "zodiac_relation": "2,5",
        "label": "Mercury",
        "label_short": "mercury"
    },
    {
        "id": 3,
        "name": "Venus",
        "color": "#400052",
        "visible": 1,
        "element_points": 15,
        "zodiac_relation": "1,6",
        "label": "Venus",
        "label_short": "venus"
    },
    {
        "id": 4,
        "name": "Mars",
        "color": "#540000",
        "visible": 1,
        "element_points": 15,
        "zodiac_relation": "0",
        "label": "Mars",
        "label_short": "mars"
    },
    {
        "id": 5,
        "name": "Jupiter",
        "color": "#47133d",
        "visible": 1,
        "element_points": 10,
        "zodiac_relation": "8",
        "label": "Jupiter",
        "label_short": "jupiter"
    },
    {
        "id": 6,
        "name": "Saturn",
        "color": "#124500",
        "visible": 1,
        "element_points": 10,
        "zodiac_relation": "9",
        "label": "Saturn",
        "label_short": "saturn"
    },
    {
        "id": 7,
        "name": "Uranus",
        "color": "#6f0766",
        "visible": 1,
        "element_points": 10,
        "zodiac_relation": "10",
        "label": "Uranus",
        "label_short": "uranus"
    },
    {
        "id": 8,
        "name": "Neptune",
        "color": "#06537f",
        "visible": 1,
        "element_points": 10,
        "zodiac_relation": "11",
        "label": "Neptune",
        "label_short": "neptune"
    },
    {
        "id": 9,
        "name": "Pluto",
        "color": "#713f04",
        "visible": 0,
        "element_points": 10,
        "zodiac_relation": "7",
        "label": "Pluto",
        "label_short": "pluto"
    },
    {
        "id": 10,
        "name": "Mean_Node",
        "color": "#4c1541",
        "visible": 0,
        "element_points": 20,
        "zodiac_relation": "-1",
        "label": "North Node",
        "label_short": "Node"
    },
    {
        "id": 11,
        "name": "True_Node",
        "color": "#4c1541",
        "visible": 0,
        "element_points": 0,
        "zodiac_relation": "-1",
        "label": "True_Node",
        "label_short": "True_Node"
    },
    {
        "id": 12,
        "name": "1",
        "color": "orange",
        "visible": 1,
        "element_points": 40,
        "zodiac_relation": "-1",
        "label": "Asc",
        "label_short": "Asc"
    },
    {
        "id": 13,
        "name": "10",
        "color": "#FF0000",
        "visible": 1,
        "element_points": 20,
        "zodiac_relation": "-1",
        "label": "Mc",
        "label_short": "Mc"
    },
    {
        "id": 14,
        "name": "7",
        "color": "#0000FF",
        "visible": 0,
        "element_points": 0,
        "zodiac_relation": "-1",
        "label": "Dsc",
        "label_short": "Dsc"
    },
    {
        "id": 15,
        "name": "4",
        "color": "#000000",
        "visible": 0,
        "element_points": 0,
        "zodiac_relation": "-1",
        "label": "Ic",
        "label_short": "Ic"
    }
]

# Axes:

axes_orbit = 1



"""
    This is part of Kerykeion (C) 2020 Giacomo Battaglia
"""

import setuptools

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kerykeion",
    version="2.2.2",
    author="Giacomo Battaglia",
    author_email="battaglia.giacomo@yahoo.it",
    description="An astrology library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/g-battaglia/kerykeion",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    python_requires='>=3.6',
    install_requires=['pyswisseph', 'pytz', 'jsonpickle'],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# * TODO: Change the label for language in 2 objects, one for IT and one for ENG.
# * Then change the label for planets like this: planet["label"] => planet[language]

# basics
# import settings as chart_settings
import kerykeion.utilities.kr_settings as chart_settings

import math
import os.path
import datetime
import json
import pytz
import kerykeion as kr

# template processing
from string import Template

# minidom parser
from xml.dom.minidom import parseString


# calculation and svg drawing class
class MakeSvgInstance:
    """
    Creates the instance that can genearte the chart with the
    function makeSVG().

    There are 2 templates, the extended (default) which has all the
    information and the basic, which has just the chart.

    Args: First kerykeion object,
    Chart type (Natal, Transit, Composite, Default: Type="Natal"),
    second kerykeion object (Not required if type is Natal)


    """

    def __init__(self, first_obj, chart_type="Natal", second_obj=None, new_output_directory=None, template_type="extended"):
    
        # Directories:
        DATADIR = os.path.dirname(__file__)
        self.homedir = os.path.expanduser("~")
        
        if new_output_directory:
            self.output_directory = new_output_directory
        else:
            self.output_directory = self.homedir
            
        self.template = False
        
        # Template types:
        if template_type == "basic":
            self.xml_svg = os.path.join(DATADIR, 'templates/basic.xml')
        else:
            self.xml_svg = os.path.join(DATADIR, 'templates/extended.xml')

        # SVG Width
        self.natal_width = 772.2
        self.full_width = 1200

        # Settings file

        self.language_settings = chart_settings.english
        self.colors_settings = chart_settings.colors
        self.planets_settings = chart_settings.planets
        self.aspects_settings = chart_settings.aspects
        self.type = chart_type

        # Kerykeion instance
        self.user = first_obj
        if not hasattr(self.user, "sun"):
            print(f"Generating kerykeion object for {self.user.name}...")
            self.user.get_all()

        # Make a list for the absolute degrees of the points of the graphic.

        self.points_deg_ut = self.user.planets_degrees + [self.user.houses_degree_ut[0],
                                                          self.user.houses_degree_ut[9], self.user.houses_degree_ut[6],
                                                          self.user.houses_degree_ut[3]]

        # Make a list of the relative degrees of the points in the graphic.

        self.points_deg = []
        for planet in self.user.planets_list:
            self.points_deg.append(planet["position"])

        self.points_deg = self.points_deg + [self.user.houses_list[0]["position"],
                                             self.user.houses_list[9]["position"], self.user.houses_list[6]["position"],
                                             self.user.houses_list[3]["position"]]

        # Make list of the poits sign.

        self.points_sign = []

        for planet in self.user.planets_list:
            self.points_sign.append(planet["sign_num"])

        self.points_sign = self.points_sign + [self.user.houses_list[0]["sign_num"],
                                               self.user.houses_list[9]["sign_num"], self.user.houses_list[6]["sign_num"],
                                               self.user.houses_list[3]["sign_num"]]

        # Make a list of poits if they are retrograde or not.

        self.points_retrograde = []

        for planet in self.user.planets_list:
            self.points_retrograde.append(planet["retrograde"])

        self.points_retrograde = self.points_retrograde + [False,
                                                           False, False, False]

        # Makes the sign number list.

        self.houses_sign_graph = []
        for h in self.user.houses_list:
            self.houses_sign_graph.append(h['sign_num'])

        if self.type == "Natal":
            natal_aspects_instance = kr.utilities.NatalAspects(self.user)
            self.aspects_list = natal_aspects_instance.get_aspects()

        if self.type == "Transit" or self.type == "Composite":

            # Kerykeion instance
            self.t_user = second_obj

            if not hasattr(self.t_user, "sun"):
                print(f"Generating kerykeion object for {self.t_user.name}...")
                self.t_user.get_all()

            # Make a list for the absolute degrees of the points of the graphic.

            self.t_points_deg_ut = self.t_user.planets_degrees + [self.t_user.houses_degree_ut[0],
                                                                  self.t_user.houses_degree_ut[
                                                                      9], self.t_user.houses_degree_ut[6],
                                                                  self.t_user.houses_degree_ut[3]]

            # Make a list of the relative degrees of the points in the graphic.

            self.t_points_deg = []
            for planet in self.t_user.planets_list:
                self.t_points_deg.append(planet["position"])

            self.t_points_deg = self.t_points_deg + [self.t_user.houses_list[0]["position"],
                                                     self.t_user.houses_list[9]["position"], self.t_user.houses_list[6]["position"],
                                                     self.t_user.houses_list[3]["position"]]

            # Make list of the poits sign.

            self.t_points_sign = []

            for planet in self.t_user.planets_list:
                self.t_points_sign.append(planet["sign_num"])

            self.t_points_sign = self.t_points_sign + [self.t_user.houses_list[0]["sign_num"],
                                                       self.t_user.houses_list[9]["sign_num"], self.t_user.houses_list[6]["sign_num"],
                                                       self.t_user.houses_list[3]["sign_num"]]

            # Make a list of poits if they are retrograde or not.

            self.t_points_retrograde = []

            for planet in self.t_user.planets_list:
                self.t_points_retrograde.append(planet["retrograde"])

            self.t_points_retrograde = self.t_points_retrograde + [False,
                                                                   False, False, False]

            self.t_houses_sign_graph = []
            for h in self.t_user.houses_list:
                self.t_houses_sign_graph.append(h['sign_num'])

        # screen size
        if self.type == "Natal":
            self.screen_width = 772.2
        else:
            self.screen_width = 1200
        self.screen_height = 772.2

        # check for home
        self.home_location = self.user.city
        self.home_geolat = self.user.city_lat
        self.home_geolon = self.user.city_long
        self.home_countrycode = self.user.nation
        self.home_timezonestr = self.user.city_tz

        print(f'{self.user.name} birth location: {self.home_location}, {self.home_geolat}, {self.home_geolon}')

        # default location
        self.location = self.home_location
        self.geolat = float(self.home_geolat)
        self.geolon = float(self.home_geolon)
        self.countrycode = self.home_countrycode
        self.timezonestr = self.home_timezonestr

        # current datetime
        now = datetime.datetime.now()

        # aware datetime object
        dt_input = datetime.datetime(
            now.year, now.month, now.day, now.hour, now.minute, now.second)
        dt = pytz.timezone(self.timezonestr).localize(dt_input)

        # naive utc datetime object
        dt_utc = dt.replace(tzinfo=None) - dt.utcoffset()

        # Default
        self.name = self.user.name
        self.charttype = self.type
        self.year = self.user.utc.year
        self.month = self.user.utc.month
        self.day = self.user.utc.day
        self.hour = self.user.utc.hour + self.user.utc.minute/100
        self.timezone = self.offsetToTz(dt.utcoffset())
        self.altitude = 25
        self.geonameid = None

        # Transit

        if self.type == "Transit":
            self.t_geolon = self.geolon
            self.t_geolat = self.geolat
            self.t_altitude = self.altitude
            self.t_name = self.language_settings['transit_name']
            self.t_year = dt_utc.year
            self.t_month = dt_utc.month
            self.t_day = dt_utc.day
            self.t_hour = self.decHourJoin(
                dt_utc.hour, dt_utc.minute, dt_utc.second)
            self.t_timezone = self.offsetToTz(dt.utcoffset())
            self.t_altitude = 25
            self.t_geonameid = None

        # configuration
        # ZOOM 1 = 100%
        self.zoom = 1

        # 12 zodiacs
        self.zodiac = ['aries', 'taurus', 'gemini', 'cancer', 'leo', 'virgo',
                       'libra', 'scorpio', 'sagittarius', 'capricorn', 'aquarius', 'pisces']
        self.zodiac_short = ['Ari', 'Tau', 'Gem', 'Cnc', 'Leo',
                             'Vir', 'Lib', 'Sco', 'Sgr', 'Cap', 'Aqr', 'Psc']
        self.zodiac_color = ['#482900', '#6b3d00', '#5995e7', '#2b4972', '#c54100',
                             '#2b286f', '#69acf1', '#ffd237', '#ff7200', '#863c00', '#4f0377', '#6cbfff']
        self.zodiac_element = ['fire', 'earth', 'air', 'water', 'fire',
                               'earth', 'air', 'water', 'fire', 'earth', 'air', 'water']

        # get color configuration

        return

    def makeTemplate(self, printing=None):
        # self.type = "Transit"
        # empty element points
        self.fire = 0.0
        self.earth = 0.0
        self.air = 0.0
        self.water = 0.0

        # Transit module data
        if self.type == "Transit" or self.type == "Composite":
            # grab transiting module data

            self.t_planets_sign = self.t_points_sign
            self.t_planets_degree = self.t_points_deg
            self.t_planets_degree_ut = self.t_points_deg_ut
            self.t_planets_retrograde = self.t_points_retrograde
            self.t_houses_list = self.t_user.houses_list
            self.t_houses_sign = self.t_houses_sign_graph
            self.t_houses_degree_ut = self.t_user.houses_degree_ut

        # grab normal module data
        self.planets_sign = self.points_sign
        self.planets_degree = self.points_deg
        self.planets_degree_ut = self.points_deg_ut
        self.planets_retrograde = self.points_retrograde
        self.houses_list = self.user.houses_list
        self.houses_sign = self.houses_sign_graph
        self.houses_degree_ut = self.user.houses_degree_ut
        self.lunar_phase = self.user.lunar_phase
        #

        # width and height from screen
        ratio = float(self.screen_width) / float(self.screen_height)
        if ratio < 1.3:  # 1280x1024
            wm_off = 130
        else:  # 1024x768, 800x600, 1280x800, 1680x1050
            wm_off = 100

        # Viewbox and sizing
        svgHeight = "100%"  # self.screen_height-wm_off
        svgWidth = "100%"  #  self.screen_width-5.0
        # svgHeight=self.screen_height-wm_off
        # svgWidth=(770.0*svgHeight)/540.0
        # svgWidth=float(self.screen_width)-25.0
        rotate = "0"
        translate = "0"
        # Defoult:
        # viewbox = '0 0 772.2 546.0' #297mm * 2.6 + 210mm * 2.6
        if self.type == "Natal":
            viewbox = '0 0 772.2 546.0'  # 297mm * 2.6 + 210mm * 2.6
        else:
            viewbox = '0 0 1000 546.0'

        # template dictionary
        td = dict()
        r = 240
        self.c1 = 0
        self.c2 = 36
        self.c3 = 120

        # transit
        if self.type == "Transit" or self.type == "Composite":
            td['transitRing'] = self.transitRing(r)
            td['degreeRing'] = self.degreeTransitRing(r)
            # circles
            td['c1'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-36) + '"'
            td['c1style'] = 'fill: none; stroke: %s; stroke-width: 1px; stroke-opacity:.4;' % (
                self.colors_settings['zodiac_transit_ring_2'])
            td['c2'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-72) + '"'
            td['c2style'] = 'fill: %s; fill-opacity:.4; stroke: %s; stroke-opacity:.4; stroke-width: 1px' % (
                self.colors_settings['paper_1'], self.colors_settings['zodiac_transit_ring_1'])
            td['c3'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-160) + '"'
            td['c3style'] = 'fill: %s; fill-opacity:.8; stroke: %s; stroke-width: 1px' % (
                self.colors_settings['paper_1'], self.colors_settings['zodiac_transit_ring_0'])
            td['makeAspects'] = self.makeAspectsTransit(r, (r-160))
            td['makeAspectGrid'] = self.makeAspectTransitGrid(r)
            td['makePatterns'] = ''
            td['chart_width'] = self.full_width
        else:
            td['transitRing'] = ""
            td['degreeRing'] = self.degreeRing(r)
            # circles
            td['c1'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-self.c1) + '"'
            td['c1style'] = 'fill: none; stroke: %s; stroke-width: 1px; ' % (
                self.colors_settings['zodiac_radix_ring_2'])
            td['c2'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-self.c2) + '"'
            td['c2style'] = 'fill: %s; fill-opacity:.2; stroke: %s; stroke-opacity:.4; stroke-width: 1px' % (
                self.colors_settings['paper_1'], self.colors_settings['zodiac_radix_ring_1'])
            td['c3'] = 'cx="' + str(r) + '" cy="' + \
                str(r) + '" r="' + str(r-self.c3) + '"'
            td['c3style'] = 'fill: %s; fill-opacity:.8; stroke: %s; stroke-width: 1px' % (
                self.colors_settings['paper_1'], self.colors_settings['zodiac_radix_ring_0'])
            td['makeAspects'] = self.makeAspects(r, (r-self.c3))
            td['makeAspectGrid'] = self.makeAspectGrid(r)
            td['makePatterns'] = self.makePatterns()
            td['chart_width'] = self.natal_width

        td['circleX'] = str(0)
        td['circleY'] = str(0)
        td['svgWidth'] = str(svgWidth)
        td['svgHeight'] = str(svgHeight)
        td['viewbox'] = viewbox
        if self.type == "Composite":
            td['stringTitle'] = f"{self.name} {self.language_settings['&']} {self.t_user.name}"
        elif self.type == "Transit":
            td['stringTitle'] = f"{self.language_settings['transits']} {self.t_user.day}/{self.t_user.month}/{self.t_user.year}"
        else:
            td['stringTitle'] = self.name

        # Tipo di carta
        if self.type == "Composite" or self.name == "Transit":
            td['stringName'] = f"{self.name}:"
        else:
            td['stringName'] = "Info:"

        # bottom left

        td['bottomLeft1'] = ''
        td['bottomLeft2'] = ''
        td['bottomLeft3'] = '%s: %s' % (("Lunar Phase"), self.dec2deg(
            self.lunar_phase['degrees_between_s_m']))
        td['bottomLeft4'] = ''

        # lunar phase
        deg = self.lunar_phase['degrees_between_s_m']

        if(deg < 90.0):
            maxr = deg
            if(deg > 80.0):
                maxr = maxr*maxr
            lfcx = 20.0+(deg/90.0)*(maxr+10.0)
            lfr = 10.0+(deg/90.0)*maxr
            lffg, lfbg = self.colors_settings["lunar_phase_0"], self.colors_settings["lunar_phase_1"]

        elif(deg < 180.0):
            maxr = 180.0-deg
            if(deg < 100.0):
                maxr = maxr*maxr
            lfcx = 20.0+((deg-90.0)/90.0*(maxr+10.0))-(maxr+10.0)
            lfr = 10.0+maxr-((deg-90.0)/90.0*maxr)
            lffg, lfbg = self.colors_settings["lunar_phase_1"], self.colors_settings["lunar_phase_0"]

        elif(deg < 270.0):
            maxr = deg-180.0
            if(deg > 260.0):
                maxr = maxr*maxr
            lfcx = 20.0+((deg-180.0)/90.0*(maxr+10.0))
            lfr = 10.0+((deg-180.0)/90.0*maxr)
            lffg, lfbg = self.colors_settings["lunar_phase_1"], self.colors_settings["lunar_phase_0"]

        elif(deg < 361):
            maxr = 360.0-deg
            if(deg < 280.0):
                maxr = maxr*maxr
            lfcx = 20.0+((deg-270.0)/90.0*(maxr+10.0))-(maxr+10.0)
            lfr = 10.0+maxr-((deg-270.0)/90.0*maxr)
            lffg, lfbg = self.colors_settings["lunar_phase_0"], self.colors_settings["lunar_phase_1"]

        td['lunar_phase_fg'] = lffg
        td['lunar_phase_bg'] = lfbg
        td['lunar_phase_cx'] = lfcx
        td['lunar_phase_r'] = lfr
        td['lunar_phase_outline'] = self.colors_settings["lunar_phase_2"]

        # rotation based on latitude
        td['lunar_phase_rotate'] = (-90.0-self.geolat)

        # stringlocation
        if len(self.location) > 35:
            split = self.location.split(",")
            if len(split) > 1:
                td['stringLocation'] = split[0]+", "+split[-1]
                if len(td['stringLocation']) > 35:
                    td['stringLocation'] = td['stringLocation'][:35]+"..."
            else:
                td['stringLocation'] = self.location[:35]+"..."
        else:
            td['stringLocation'] = self.location

        # td['stringDateTime']= str(self.user.year)+'-%(#1)02d-%(#2)02d %(#3)02d:%(#4)02d:%(#5)02d' % {'#1':self.user.month,'#2':self.user.day,'#3':self.user.hours,'#4':self.user.minuts,'#5':00} + self.decTzStr(self.timezone)
        td['stringDateTime'] = f'{self.user.year}-{self.user.month}-{self.user.day} {self.user.hours:02d}:{self.user.minuts:02d}'

        if self.type == "Composite":
            td['stringLat'] = f'{self.t_user.name}: '
            td['stringLon'] = self.t_user.city
            td['stringPosition'] = f'{self.t_user.year}-{self.t_user.month}-{self.t_user.day} {self.t_user.hours:02d}:{self.t_user.minuts:02d}'

        else:
            td['stringLat'] = "%s: %s" % (
                self.language_settings['latitude'], self.lat2str(self.geolat))
            td['stringLon'] = "%s: %s" % (
                self.language_settings['longitude'], self.lon2str(self.geolon))
            td['stringPosition'] = f"{self.language_settings['type']}: {self.charttype}"

        # paper_color_X
        td['paper_color_0'] = self.colors_settings["paper_0"]
        td['paper_color_1'] = self.colors_settings["paper_1"]

        # planets_color_X
        for i in range(len(self.planets_settings)):
            td['planets_color_%s' %
                (i)] = self.colors_settings["planet_%s" % (i)]

        # zodiac_color_X
        for i in range(12):
            td['zodiac_color_%s' %
                (i)] = self.colors_settings["zodiac_icon_%s" % (i)]

        # orb_color_X
        for i in range(len(self.aspects_settings)):
            td['orb_color_%s' % (self.aspects_settings[i]['degree'])] = self.colors_settings["aspect_%s" % (
                self.aspects_settings[i]['degree'])]

        # config
        td['cfgZoom'] = str(self.zoom)
        td['cfgRotate'] = rotate
        td['cfgTranslate'] = translate

        # functions
        td['makeZodiac'] = self.makeZodiac(r)
        td['makeHouses'] = self.makeHouses(r)
        td['makePlanets'] = self.makePlanets(r)
        td['makeElements'] = self.makeElements(r)
        td['makePlanetGrid'] = self.makePlanetGrid()
        td['makeHousesGrid'] = self.makeHousesGrid()

        # read template
        with open(self.xml_svg, "r") as output_file:
            f = open(self.xml_svg)
            self.template = Template(f.read()).substitute(td)

        # return filename

        return self.template

    def makeSVG(self):
        """Prints out the SVG file in the specifide folder"""

        if not (self.template):
            self.makeTemplate()

        self.chartname = os.path.join(
            self.output_directory, f'{self.name}{self.type}Chart.svg')

        with open(self.chartname, "w") as output_file:
            output_file.write(self.template)

        return print(f"SVG Generated Correctly in: {self.output_directory}")

    # draw transit ring
    def transitRing(self, r):
        out = '<circle cx="%s" cy="%s" r="%s" style="fill: none; stroke: %s; stroke-width: 36px; stroke-opacity: .4;"/>' % (
            r, r, r-18, self.colors_settings['paper_1'])
        out += '<circle cx="%s" cy="%s" r="%s" style="fill: none; stroke: %s; stroke-width: 1px; stroke-opacity: .6;"/>' % (
            r, r, r, self.colors_settings['zodiac_transit_ring_3'])
        return out

    # draw degree ring
    def degreeRing(self, r):
        out = ''
        for i in range(72):
            offset = float(i*5) - self.houses_degree_ut[6]
            if offset < 0:
                offset = offset + 360.0
            elif offset > 360:
                offset = offset - 360.0
            x1 = self.sliceToX(0, r-self.c1, offset) + self.c1
            y1 = self.sliceToY(0, r-self.c1, offset) + self.c1
            x2 = self.sliceToX(0, r+2-self.c1, offset) - 2 + self.c1
            y2 = self.sliceToY(0, r+2-self.c1, offset) - 2 + self.c1
            out += '<line x1="%s" y1="%s" x2="%s" y2="%s" style="stroke: %s; stroke-width: 1px; stroke-opacity:.9;"/>\n' % (
                x1, y1, x2, y2, self.colors_settings['paper_0'])
        return out

    def degreeTransitRing(self, r):
        out = ''
        for i in range(72):
            offset = float(i*5) - self.houses_degree_ut[6]
            if offset < 0:
                offset = offset + 360.0
            elif offset > 360:
                offset = offset - 360.0
            x1 = self.sliceToX(0, r, offset)
            y1 = self.sliceToY(0, r, offset)
            x2 = self.sliceToX(0, r+2, offset) - 2
            y2 = self.sliceToY(0, r+2, offset) - 2
            out += '<line x1="%s" y1="%s" x2="%s" y2="%s" style="stroke: #F00; stroke-width: 1px; stroke-opacity:.9;"/>\n' % (
                x1, y1, x2, y2)
        return out

    # floating latitude an longitude to string
    def lat2str(self, coord):
        sign = self.language_settings["north"]
        if coord < 0.0:
            sign = self.language_settings["south"]
            coord = abs(coord)
        deg = int(coord)
        min = int((float(coord) - deg) * 60)
        sec = int(round(float(((float(coord) - deg) * 60) - min) * 60.0))
        return "%s°%s'%s\" %s" % (deg, min, sec, sign)

    def lon2str(self, coord):
        sign = self.language_settings["east"]
        if coord < 0.0:
            sign = self.language_settings["west"]
            coord = abs(coord)
        deg = int(coord)
        min = int((float(coord) - deg) * 60)
        sec = int(round(float(((float(coord) - deg) * 60) - min) * 60.0))
        return "%s°%s'%s\" %s" % (deg, min, sec, sign)

    # decimal hour to minutes and seconds
    def decHour(self, input):
        hours = int(input)
        mands = (input-hours)*60.0
        mands = round(mands, 5)
        minutes = int(mands)
        seconds = int(round((mands-minutes)*60))
        return [hours, minutes, seconds]

    # join hour, minutes, seconds, timezone integere to hour float
    def decHourJoin(self, inH, inM, inS):
        dh = float(inH)
        dm = float(inM)/60
        ds = float(inS)/3600
        output = dh + dm + ds
        return output

    # Datetime offset to float in hours
    def offsetToTz(self, dtoffset):
        dh = float(dtoffset.days * 24)
        sh = float(dtoffset.seconds / 3600.0)
        output = dh + sh
        return output

    # decimal timezone string
    def decTzStr(self, tz):
        if tz > 0:
            h = int(tz)
            m = int((float(tz)-float(h))*float(60))
            return " [+%(#1)02d:%(#2)02d]" % {'#1': h, '#2': m}
        else:
            h = int(tz)
            m = int((float(tz)-float(h))*float(60))/-1
            return " [-%(#1)02d:%(#2)02d]" % {'#1': h/-1, '#2': m}

    # degree difference
    def degreeDiff(self, a, b):
        out = float()
        if a > b:
            out = a - b
        if a < b:
            out = b-a
        if out > 180.0:
            out = 360.0-out
        return out

    # decimal to degrees (a°b'c")
    def dec2deg(self, dec, type="3"):
        dec = float(dec)
        a = int(dec)
        a_new = (dec-float(a)) * 60.0
        b_rounded = int(round(a_new))
        b = int(a_new)
        c = int(round((a_new-float(b))*60.0))
        if type == "3":
            out = '%(#1)02d&#176;%(#2)02d&#39;%(#3)02d&#34;' % {
                '#1': a, '#2': b, '#3': c}
        elif type == "2":
            out = '%(#1)02d&#176;%(#2)02d&#39;' % {'#1': a, '#2': b_rounded}
        elif type == "1":
            out = '%(#1)02d&#176;' % {'#1': a}
        return str(out)

    # draw svg aspects: ring, aspect ring, degreeA degreeB
    def drawAspect(self, r, ar, degA, degB, color):
        offset = (int(self.houses_degree_ut[6]) / -1) + int(degA)
        x1 = self.sliceToX(0, ar, offset) + (r-ar)
        y1 = self.sliceToY(0, ar, offset) + (r-ar)
        offset = (int(self.houses_degree_ut[6]) / -1) + int(degB)
        x2 = self.sliceToX(0, ar, offset) + (r-ar)
        y2 = self.sliceToY(0, ar, offset) + (r-ar)
        out = '            <line x1="'+str(x1)+'" y1="'+str(y1)+'" x2="'+str(x2)+'" y2="'+str(
            y2)+'" style="stroke: '+color+'; stroke-width: 1; stroke-opacity: .9;"/>\n'
        return out

    def sliceToX(self, slice, r, offset):
        plus = (math.pi * offset) / 180
        radial = ((math.pi/6) * slice) + plus
        return r * (math.cos(radial)+1)

    def sliceToY(self, slice, r, offset):
        plus = (math.pi * offset) / 180
        radial = ((math.pi/6) * slice) + plus
        return r * ((math.sin(radial)/-1)+1)

    def zodiacSlice(self, num, r, style,  type):
        # pie slices
        offset = 360 - self.houses_degree_ut[6]
        # check transit
        if self.type == "Transit" or self.type == "Composite":
            dropin = 0
        else:
            dropin = self.c1
        slice = '<path d="M' + str(r) + ',' + str(r) + ' L' + str(dropin + self.sliceToX(num, r-dropin, offset)) + ',' + str(dropin + self.sliceToY(num, r-dropin, offset)) + ' A' + str(
            r-dropin) + ',' + str(r-dropin) + ' 0 0,0 ' + str(dropin + self.sliceToX(num+1, r-dropin, offset)) + ',' + str(dropin + self.sliceToY(num+1, r-dropin, offset)) + ' z" style="' + style + '"/>'
        # symbols
        offset = offset + 15
        # check transit
        if self.type == "Transit" or self.type == "Composite":
            dropin = 54
        else:
            dropin = 18+self.c1
        sign = '<g transform="translate(-16,-16)"><use x="' + str(dropin + self.sliceToX(num, r-dropin, offset)) + '" y="' + str(
            dropin + self.sliceToY(num, r-dropin, offset)) + '" xlink:href="#' + type + '" /></g>\n'
        return slice + '\n' + sign

    def makeZodiac(self, r):
        output = ""
        for i in range(len(self.zodiac)):
            output = output + self.zodiacSlice(i, r, "fill:" + self.colors_settings["zodiac_bg_%s" % (
                i)] + "; fill-opacity: 0.5;", self.zodiac[i]) + '\n'
        return output

    def makeHouses(self, r):
        path = ""

        xr = 12
        for i in range(xr):
            # check transit
            if self.type == "Transit" or self.type == "Composite":
                dropin = 160
                roff = 72
                t_roff = 36
            else:
                dropin = self.c3
                roff = self.c1

            # offset is negative desc houses_degree_ut[6]
            offset = (
                int(self.houses_degree_ut[int(xr/2)]) / -1) + int(self.houses_degree_ut[i])
            x1 = self.sliceToX(0, (r-dropin), offset) + dropin
            y1 = self.sliceToY(0, (r-dropin), offset) + dropin
            x2 = self.sliceToX(0, r-roff, offset) + roff
            y2 = self.sliceToY(0, r-roff, offset) + roff

            if i < (xr-1):
                text_offset = offset + \
                    int(self.degreeDiff(self.houses_degree_ut[(
                        i+1)], self.houses_degree_ut[i]) / 2)
            else:
                text_offset = offset + \
                    int(self.degreeDiff(
                        self.houses_degree_ut[0], self.houses_degree_ut[(xr-1)]) / 2)

            # mc, asc, dsc, ic
            if i == 0:
                linecolor = self.planets_settings[12]['color']
            elif i == 9:
                linecolor = self.planets_settings[13]['color']
            elif i == 6:
                linecolor = self.planets_settings[14]['color']
            elif i == 3:
                linecolor = self.planets_settings[15]['color']
            else:
                linecolor = self.colors_settings['houses_radix_line']

            # Transit houses lines.
            if self.type == "Transit" or self.type == "Composite":

                # Degrees for point zero.

                zeropoint = 360 - self.houses_degree_ut[6]
                t_offset = zeropoint + self.t_houses_degree_ut[i]
                if t_offset > 360:
                    t_offset = t_offset - 360
                t_x1 = self.sliceToX(0, (r-t_roff), t_offset) + t_roff
                t_y1 = self.sliceToY(0, (r-t_roff), t_offset) + t_roff
                t_x2 = self.sliceToX(0, r, t_offset)
                t_y2 = self.sliceToY(0, r, t_offset)
                if i < 11:
                    t_text_offset = t_offset + \
                        int(self.degreeDiff(self.t_houses_degree_ut[(
                            i+1)], self.t_houses_degree_ut[i]) / 2)
                else:
                    t_text_offset = t_offset + \
                        int(self.degreeDiff(
                            self.t_houses_degree_ut[0], self.t_houses_degree_ut[11]) / 2)
                # linecolor
                if i == 0 or i == 9 or i == 6 or i == 3:
                    t_linecolor = linecolor
                else:
                    t_linecolor = self.colors_settings['houses_transit_line']
                xtext = self.sliceToX(0, (r-8), t_text_offset) + 8
                ytext = self.sliceToY(0, (r-8), t_text_offset) + 8

                if self.type == "Transit":
                    path = path + '<text style="fill: #00f; fill-opacity: 0; font-size: 14px"><tspan x="' + \
                        str(xtext-3)+'" y="'+str(ytext+3) + \
                        '">'+str(i+1)+'</tspan></text>\n'
                    path = path + '<line x1="'+str(t_x1)+'" y1="'+str(t_y1)+'" x2="'+str(t_x2)+'" y2="'+str(
                        t_y2)+'" style="stroke: '+t_linecolor+'; stroke-width: 2px; stroke-opacity:0;"/>\n'

                else:
                    path = path + '<text style="fill: #00f; fill-opacity: .4; font-size: 14px"><tspan x="' + \
                        str(xtext-3)+'" y="'+str(ytext+3) + \
                        '">'+str(i+1)+'</tspan></text>\n'
                    path = path + '<line x1="'+str(t_x1)+'" y1="'+str(t_y1)+'" x2="'+str(t_x2)+'" y2="'+str(
                        t_y2)+'" style="stroke: '+t_linecolor+'; stroke-width: 2px; stroke-opacity:.3;"/>\n'

            # if transit
            if self.type == "Transit" or self.type == "Composite":
                dropin = 84

            dropin = 48

            xtext = self.sliceToX(
                0, (r-dropin), text_offset) + dropin  # was 132
            ytext = self.sliceToY(
                0, (r-dropin), text_offset) + dropin  # was 132
            path = path + '<line x1="'+str(x1)+'" y1="'+str(y1)+'" x2="'+str(x2)+'" y2="'+str(
                y2)+'" style="stroke: '+linecolor+'; stroke-width: 2px; stroke-dasharray:3,2; stroke-opacity:.4;"/>\n'
            path = path + '<text style="fill: #f00; fill-opacity: .6; font-size: 14px"><tspan x="' + \
                str(xtext-3)+'" y="'+str(ytext+3) + \
                '">'+str(i+1)+'</tspan></text>\n'

        return path

    def makePlanets(self, r):

        planets_degut = {}

        diff = range(len(self.planets_settings))
        for i in range(len(self.planets_settings)):
            if self.planets_settings[i]['visible'] == 1:
                # list of planets sorted by degree
                planets_degut[self.planets_degree_ut[i]] = i

            # element: get extra points if planet is in own zodiac
            pz = self.planets_settings[i]['zodiac_relation']
            cz = self.planets_sign[i]
            extrapoints = 0
            if pz != -1:
                for e in range(len(pz.split(','))):
                    if int(pz.split(',')[e]) == int(cz):
                        extrapoints = 10

            # calculate element points for all planets
            ele = self.zodiac_element[self.planets_sign[i]]
            if ele == "fire":
                self.fire = self.fire + \
                    self.planets_settings[i]['element_points'] + extrapoints
            elif ele == "earth":
                self.earth = self.earth + \
                    self.planets_settings[i]['element_points'] + extrapoints
            elif ele == "air":
                self.air = self.air + \
                    self.planets_settings[i]['element_points'] + extrapoints
            elif ele == "water":
                self.water = self.water + \
                    self.planets_settings[i]['element_points'] + extrapoints

        output = ""
        keys = list(planets_degut.keys())
        keys.sort()
        switch = 0

        planets_degrouped = {}
        groups = []
        planets_by_pos = list(range(len(planets_degut)))
        planet_drange = 3.4
        # get groups closely together
        group_open = False
        for e in range(len(keys)):
            i = planets_degut[keys[e]]
            # get distances between planets
            if e == 0:
                prev = self.planets_degree_ut[planets_degut[keys[-1]]]
                next = self.planets_degree_ut[planets_degut[keys[1]]]
            elif e == (len(keys)-1):
                prev = self.planets_degree_ut[planets_degut[keys[e-1]]]
                next = self.planets_degree_ut[planets_degut[keys[0]]]
            else:
                prev = self.planets_degree_ut[planets_degut[keys[e-1]]]
                next = self.planets_degree_ut[planets_degut[keys[e+1]]]
            diffa = self.degreeDiff(prev, self.planets_degree_ut[i])
            diffb = self.degreeDiff(next, self.planets_degree_ut[i])
            planets_by_pos[e] = [i, diffa, diffb]
            # print "%s %s %s" % (self.planets_settings[i]['label'],diffa,diffb)
            if (diffb < planet_drange):
                if group_open:
                    groups[-1].append([e, diffa, diffb,
                                      self.planets_settings[planets_degut[keys[e]]]["label"]])
                else:
                    group_open = True
                    groups.append([])
                    groups[-1].append([e, diffa, diffb,
                                      self.planets_settings[planets_degut[keys[e]]]["label"]])
            else:
                if group_open:
                    groups[-1].append([e, diffa, diffb,
                                      self.planets_settings[planets_degut[keys[e]]]["label"]])
                group_open = False

        def zero(x): return 0
        planets_delta = list(map(zero, range(len(self.planets_settings))))

        # print groups
        # print planets_by_pos
        for a in range(len(groups)):
            # Two grouped planets
            if len(groups[a]) == 2:
                next_to_a = groups[a][0][0]-1
                if groups[a][1][0] == (len(planets_by_pos)-1):
                    next_to_b = 0
                else:
                    next_to_b = groups[a][1][0]+1
                # if both planets have room
                if (groups[a][0][1] > (2*planet_drange)) & (groups[a][1][2] > (2*planet_drange)):
                    planets_delta[groups[a][0][0]] = - \
                        (planet_drange-groups[a][0][2])/2
                    planets_delta[groups[a][1][0]] = + \
                        (planet_drange-groups[a][0][2])/2
                # if planet a has room
                elif (groups[a][0][1] > (2*planet_drange)):
                    planets_delta[groups[a][0][0]] = -planet_drange
                # if planet b has room
                elif (groups[a][1][2] > (2*planet_drange)):
                    planets_delta[groups[a][1][0]] = +planet_drange

                # if planets next to a and b have room move them
                elif (planets_by_pos[next_to_a][1] > (2.4*planet_drange)) & (planets_by_pos[next_to_b][2] > (2.4*planet_drange)):
                    planets_delta[(next_to_a)] = (
                        groups[a][0][1]-planet_drange*2)
                    planets_delta[groups[a][0][0]] = -planet_drange*.5
                    planets_delta[next_to_b] = - \
                        (groups[a][1][2]-planet_drange*2)
                    planets_delta[groups[a][1][0]] = +planet_drange*.5

                # if planet next to a has room move them
                elif (planets_by_pos[next_to_a][1] > (2*planet_drange)):
                    planets_delta[(next_to_a)] = (
                        groups[a][0][1]-planet_drange*2.5)
                    planets_delta[groups[a][0][0]] = -planet_drange*1.2

                # if planet next to b has room move them
                elif (planets_by_pos[next_to_b][2] > (2*planet_drange)):
                    planets_delta[next_to_b] = - \
                        (groups[a][1][2]-planet_drange*2.5)
                    planets_delta[groups[a][1][0]] = +planet_drange*1.2

            # Three grouped planets or more
            xl = len(groups[a])
            if xl >= 3:

                available = groups[a][0][1]
                for f in range(xl):
                    available += groups[a][f][2]
                need = (3*planet_drange)+(1.2*(xl-1)*planet_drange)
                leftover = available - need
                xa = groups[a][0][1]
                xb = groups[a][(xl-1)][2]

                # center
                if (xa > (need*.5)) & (xb > (need*.5)):
                    startA = xa - (need*.5)
                # position relative to next planets
                else:
                    startA = (leftover/(xa+xb))*xa
                    startB = (leftover/(xa+xb))*xb

                if available > need:
                    planets_delta[groups[a][0][0]] = startA - \
                        groups[a][0][1]+(1.5*planet_drange)
                    for f in range(xl-1):
                        planets_delta[groups[a][(
                            f+1)][0]] = 1.2*planet_drange+planets_delta[groups[a][f][0]]-groups[a][f][2]

        for e in range(len(keys)):
            i = planets_degut[keys[e]]

            # coordinates
            if self.type == "Transit" or self.type == "Composite":
                if 22 < i < 27:
                    rplanet = 76
                elif switch == 1:
                    rplanet = 110
                    switch = 0
                else:
                    rplanet = 130
                    switch = 1
            else:
                # if 22 < i < 27 it is asc,mc,dsc,ic (angles of chart)
                # put on special line (rplanet is range from outer ring)
                amin, bmin, cmin = 0, 0, 0

                if 22 < i < 27:
                    rplanet = 40-cmin
                elif switch == 1:
                    rplanet = 74-amin
                    switch = 0
                else:
                    rplanet = 94-bmin
                    switch = 1

            rtext = 45

            offset = (int(self.houses_degree_ut[6]) / -1) + \
                int(self.planets_degree_ut[i]+planets_delta[e])
            trueoffset = (
                int(self.houses_degree_ut[6]) / -1) + int(self.planets_degree_ut[i])

            planet_x = self.sliceToX(0, (r-rplanet), offset) + rplanet
            planet_y = self.sliceToY(0, (r-rplanet), offset) + rplanet
            if self.type == "Transit" or self.type == "Composite":
                scale = 0.8

            scale = 1
            # output planet
            output = output + '<g transform="translate(-'+str(12*scale)+',-'+str(12*scale)+')"><g transform="scale('+str(scale)+')"><use x="' + str(
                planet_x*(1/scale)) + '" y="' + str(planet_y*(1/scale)) + '" xlink:href="#' + self.planets_settings[i]['name'] + '" /></g></g>\n'

        # make transit degut and display planets
        if self.type == "Transit" or self.type == "Composite":
            group_offset = {}
            t_planets_degut = {}
            if self.type == "Transit":
                list_range = len(self.planets_settings)-4
            else:
                list_range = len(self.planets_settings)
            for i in range(list_range):
                group_offset[i] = 0
                if self.planets_settings[i]['visible'] == 1:
                    t_planets_degut[self.t_planets_degree_ut[i]] = i
            t_keys = list(t_planets_degut.keys())
            t_keys.sort()

            # grab closely grouped planets
            groups = []
            in_group = False
            for e in range(len(t_keys)):
                i_a = t_planets_degut[t_keys[e]]
                if e == (len(t_keys)-1):
                    i_b = t_planets_degut[t_keys[0]]
                else:
                    i_b = t_planets_degut[t_keys[e+1]]

                a = self.t_planets_degree_ut[i_a]
                b = self.t_planets_degree_ut[i_b]
                diff = self.degreeDiff(a, b)
                if diff <= 2.5:
                    if in_group:
                        groups[-1].append(i_b)
                    else:
                        groups.append([i_a])
                        groups[-1].append(i_b)
                        in_group = True
                else:
                    in_group = False
            # loop groups and set degrees display adjustment
            for i in range(len(groups)):
                if len(groups[i]) == 2:
                    group_offset[groups[i][0]] = -1.0
                    group_offset[groups[i][1]] = 1.0
                elif len(groups[i]) == 3:
                    group_offset[groups[i][0]] = -1.5
                    group_offset[groups[i][1]] = 0
                    group_offset[groups[i][2]] = 1.5
                elif len(groups[i]) == 4:
                    group_offset[groups[i][0]] = -2.0
                    group_offset[groups[i][1]] = -1.0
                    group_offset[groups[i][2]] = 1.0
                    group_offset[groups[i][3]] = 2.0

            switch = 0
            for e in range(len(t_keys)):
                i = t_planets_degut[t_keys[e]]

                if 22 < i < 27:
                    rplanet = 9
                elif switch == 1:
                    rplanet = 18
                    switch = 0
                else:
                    rplanet = 26
                    switch = 1

                zeropoint = 360 - self.houses_degree_ut[6]
                t_offset = zeropoint + self.t_planets_degree_ut[i]
                if t_offset > 360:
                    t_offset = t_offset - 360
                planet_x = self.sliceToX(0, (r-rplanet), t_offset) + rplanet
                planet_y = self.sliceToY(0, (r-rplanet), t_offset) + rplanet
                output = output + '<g transform="translate(-6,-6)"><g transform="scale(0.5)"><use x="' + str(
                    planet_x*2) + '" y="' + str(planet_y*2) + '" xlink:href="#' + self.planets_settings[i]['name'] + '" /></g></g>\n'
                # transit planet line
                x1 = self.sliceToX(0, r+3, t_offset) - 3
                y1 = self.sliceToY(0, r+3, t_offset) - 3
                x2 = self.sliceToX(0, r-3, t_offset) + 3
                y2 = self.sliceToY(0, r-3, t_offset) + 3
                output = output + '<line x1="'+str(x1)+'" y1="'+str(y1)+'" x2="'+str(x2)+'" y2="'+str(
                    y2)+'" style="stroke: '+self.planets_settings[i]['color']+'; stroke-width: 1px; stroke-opacity:.8;"/>\n'

                # transit planet degree text
                rotate = self.houses_degree_ut[0] - self.t_planets_degree_ut[i]
                textanchor = "end"
                t_offset += group_offset[i]
                rtext = -3.0

                if -90 > rotate > -270:
                    rotate = rotate + 180.0
                    textanchor = "start"
                if 270 > rotate > 90:
                    rotate = rotate - 180.0
                    textanchor = "start"

                if textanchor == "end":
                    xo = 1
                else:
                    xo = -1
                deg_x = self.sliceToX(0, (r-rtext), t_offset + xo) + rtext
                deg_y = self.sliceToY(0, (r-rtext), t_offset + xo) + rtext
                degree = int(t_offset)
                output += '<g transform="translate(%s,%s)">' % (deg_x, deg_y)
                output += '<text transform="rotate(%s)" text-anchor="%s' % (
                    rotate, textanchor)
                output += '" style="fill: ' + \
                    self.planets_settings[i]['color']+'; font-size: 10px;">' + \
                    self.dec2deg(self.t_planets_degree[i], type="1")
                output += '</text></g>\n'

            # check transit
            if self.type == "Transit" or self.type == "Composite":
                dropin = 36
            else:
                dropin = 0

            # planet line
            x1 = self.sliceToX(0, r-(dropin+3), offset) + (dropin+3)
            y1 = self.sliceToY(0, r-(dropin+3), offset) + (dropin+3)
            x2 = self.sliceToX(0, (r-(dropin-3)), offset) + (dropin-3)
            y2 = self.sliceToY(0, (r-(dropin-3)), offset) + (dropin-3)
            output = output + '<line x1="'+str(x1)+'" y1="'+str(y1)+'" x2="'+str(x2)+'" y2="'+str(
                y2)+'" style="stroke: '+self.planets_settings[i]['color']+'; stroke-width: 2px; stroke-opacity:.6;"/>\n'

            # check transit
            if self.type == "Transit" or self.type == "Composite":
                dropin = 160
            else:
                dropin = 120

            x1 = self.sliceToX(0, r-dropin, offset) + dropin
            y1 = self.sliceToY(0, r-dropin, offset) + dropin
            x2 = self.sliceToX(0, (r-(dropin-3)), offset) + (dropin-3)
            y2 = self.sliceToY(0, (r-(dropin-3)), offset) + (dropin-3)
            output = output + '<line x1="'+str(x1)+'" y1="'+str(y1)+'" x2="'+str(x2)+'" y2="'+str(
                y2)+'" style="stroke: '+self.planets_settings[i]['color']+'; stroke-width: 2px; stroke-opacity:.6;"/>\n'

        return output

    def makePatterns(self):
        """
        * Stellium: At least four planets linked together in a series of continuous conjunctions.
        * Grand trine: Three trine aspects together.
        * Grand cross: Two pairs of opposing planets squared to each other.
        * T-Square: Two planets in opposition squared to a third.
        * Yod: Two qunicunxes together joined by a sextile.
        """
        conj = {}  # 0
        opp = {}  # 10
        sq = {}  # 5
        tr = {}  # 6
        qc = {}  # 9
        sext = {}  # 3
        for i in range(len(self.planets_settings)):
            a = self.planets_degree_ut[i]
            qc[i] = {}
            sext[i] = {}
            opp[i] = {}
            sq[i] = {}
            tr[i] = {}
            conj[i] = {}
            # skip some points
            n = self.planets_settings[i]['name']
            if n == 'earth' or n == 'True_Node' or n == 'osc. apogee' or n == 'intp. apogee' or n == 'intp. perigee':
                continue
            if n == 'Dsc' or n == 'Ic':
                continue
            for j in range(len(self.planets_settings)):
                # skip some points
                n = self.planets_settings[j]['name']
                if n == 'earth' or n == 'True_Node' or n == 'osc. apogee' or n == 'intp. apogee' or n == 'intp. perigee':
                    continue
                if n == 'Dsc' or n == 'Ic':
                    continue
                b = self.planets_degree_ut[j]
                delta = float(self.degreeDiff(a, b))
                # check for opposition
                xa = float(self.aspects_settings[10]['degree']) - \
                    float(self.aspects_settings[10]['orb'])
                xb = float(self.aspects_settings[10]['degree']) + \
                    float(self.aspects_settings[10]['orb'])
                if(xa <= delta <= xb):
                    opp[i][j] = True
                # check for conjunction
                xa = float(self.aspects_settings[0]['degree']) - \
                    float(self.aspects_settings[0]['orb'])
                xb = float(self.aspects_settings[0]['degree']) + \
                    float(self.aspects_settings[0]['orb'])
                if(xa <= delta <= xb):
                    conj[i][j] = True
                # check for squares
                xa = float(self.aspects_settings[5]['degree']) - \
                    float(self.aspects_settings[5]['orb'])
                xb = float(self.aspects_settings[5]['degree']) + \
                    float(self.aspects_settings[5]['orb'])
                if(xa <= delta <= xb):
                    sq[i][j] = True
                # check for qunicunxes
                xa = float(self.aspects_settings[9]['degree']) - \
                    float(self.aspects_settings[9]['orb'])
                xb = float(self.aspects_settings[9]['degree']) + \
                    float(self.aspects_settings[9]['orb'])
                if(xa <= delta <= xb):
                    qc[i][j] = True
                # check for sextiles
                xa = float(self.aspects_settings[3]['degree']) - \
                    float(self.aspects_settings[3]['orb'])
                xb = float(self.aspects_settings[3]['degree']) + \
                    float(self.aspects_settings[3]['orb'])
                if(xa <= delta <= xb):
                    sext[i][j] = True

        yot = {}
        # check for double qunicunxes
        for k, v in qc.items():
            if len(qc[k]) >= 2:
                # check for sextile
                for l, w in qc[k].items():
                    for m, x in qc[k].items():
                        if m in sext[l]:
                            if l > m:
                                yot['%s,%s,%s' % (k, m, l)] = [k, m, l]
                            else:
                                yot['%s,%s,%s' % (k, l, m)] = [k, l, m]
        tsquare = {}
        # check for opposition
        for k, v in opp.items():
            if len(opp[k]) >= 1:
                # check for square
                for l, w in opp[k].items():
                    for a, b in sq.items():
                        if k in sq[a] and l in sq[a]:
                            # print 'got tsquare %s %s %s' % (a,k,l)
                            if k > l:
                                tsquare['%s,%s,%s' % (a, l, k)] = '%s => %s, %s' % (
                                    self.planets_settings[a]['label'], self.planets_settings[l]['label'], self.planets_settings[k]['label'])
                            else:
                                tsquare['%s,%s,%s' % (a, k, l)] = '%s => %s, %s' % (
                                    self.planets_settings[a]['label'], self.planets_settings[k]['label'], self.planets_settings[l]['label'])
        stellium = {}
        # check for 4 continuous conjunctions
        for k, v in conj.items():
            if len(conj[k]) >= 1:
                # first conjunction
                for l, m in conj[k].items():
                    if len(conj[l]) >= 1:
                        for n, o in conj[l].items():
                            # skip 1st conj
                            if n == k:
                                continue
                            if len(conj[n]) >= 1:
                                # third conjunction
                                for p, q in conj[n].items():
                                    # skip first and second conj
                                    if p == k or p == n:
                                        continue
                                    if len(conj[p]) >= 1:
                                        # fourth conjunction
                                        for r, s in conj[p].items():
                                            # skip conj 1,2,3
                                            if r == k or r == n or r == p:
                                                continue

                                            l = [k, n, p, r]
                                            l.sort()
                                            stellium['%s %s %s %s' % (l[0], l[1], l[2], l[3])] = '%s %s %s %s' % (
                                                self.planets_settings[l[0]
                                                                      ]['label'], self.planets_settings[l[1]]['label'],
                                                self.planets_settings[l[2]]['label'], self.planets_settings[l[3]]['label'])
        # print yots
        out = '<g transform="translate(-30,380)">'
        if len(yot) >= 1:
            y = 0
            for k, v in yot.items():
                out += '<text y="%s" style="fill:%s; font-size: 12px;">%s</text>\n' % (
                    y, self.colors_settings['paper_0'], ("Yot"))

                # first planet symbol
                out += '<g transform="translate(20,%s)">' % (y)
                out += '<use transform="scale(0.4)" x="0" y="-20" xlink:href="#%s" /></g>\n' % (
                    self.planets_settings[yot[k][0]]['name'])

                # second planet symbol
                out += '<g transform="translate(30,%s)">' % (y)
                out += '<use transform="scale(0.4)" x="0" y="-20" xlink:href="#%s" /></g>\n' % (
                    self.planets_settings[yot[k][1]]['name'])

                # third planet symbol
                out += '<g transform="translate(40,%s)">' % (y)
                out += '<use transform="scale(0.4)" x="0" y="-20" xlink:href="#%s" /></g>\n' % (
                    self.planets_settings[yot[k][2]]['name'])

                y = y+14
        # finalize
        out += '</g>'
        # return out
        return ''

    # Aspect and aspect grid functions for natal type charts.

    def makeAspects(self, r, ar):
        out = ""
        for element in self.aspects_list:
            out = out + self.drawAspect(r, ar, element['p1_abs_pos'], element['p2_abs_pos'],
                                        self.colors_settings[f"aspect_{element['aspect_degrees']}"])

        return out

    def makeAspectGrid(self, r):

        out = ""
        style = 'stroke:%s; stroke-width: 1px; stroke-opacity:.6; fill:none' % (
            self.colors_settings['paper_0'])
        xindent = 380
        yindent = 468
        box = 14
        revr = list(range(len(self.planets_settings)))
        revr.reverse()
        counter = 0
        for a in revr:
            counter += 1
            if self.planets_settings[a]['visible'] == 1:
                out = out + '<rect x="'+str(xindent)+'" y="'+str(yindent)+'" width="'+str(
                    box)+'" height="'+str(box)+'" style="'+style+'"/>\n'
                out = out + '<use transform="scale(0.4)" x="'+str((xindent+2)*2.5)+'" y="'+str(
                    (yindent+1)*2.5)+'" xlink:href="#'+self.planets_settings[a]['name']+'" />\n'
                xindent = xindent + box
                yindent = yindent - box
                revr2 = list(range(a))
                revr2.reverse()
                xorb = xindent
                yorb = yindent + box
                for b in revr2:
                    if self.planets_settings[b]['visible'] == 1:
                        out = out + '<rect x="'+str(xorb)+'" y="'+str(yorb)+'" width="'+str(
                            box)+'" height="'+str(box)+'" style="'+style+'"/>\n'
                        xorb = xorb+box
                        for element in self.aspects_list:
                            if (element['p1'] == a and element['p2'] == b) or (element['p1'] == b and element['p2'] == a):
                                out = out + '<use  x="'+str(xorb-box+1)+'" y="'+str(
                                    yorb+1)+'" xlink:href="#orb'+str(element['aspect_degrees'])+'" />\n'

        return out

    # Aspect and aspect grid functions for transit type charts.

    def makeAspectsTransit(self, r, ar):
        out = ""

        self.aspects_list = kr.utilities.CompositeAspects(
            self.user, self.t_user).get_aspects()
        for element in self.aspects_list:
            out = out + self.drawAspect(r, ar, element['p1_abs_pos'], element['p2_abs_pos'],
                                        self.colors_settings[f"aspect_{element['aspect_degrees']}"])

        return out

    def makeAspectTransitGrid(self, r):
        out = '<g transform="translate(500,310)">'
        out += '<text y="-15" x="0" style="fill:%s; font-size: 14px;">%s</text>\n' % (
            self.colors_settings['paper_0'], (f"{self.language_settings['aspects']}:"))
        line = 0
        nl = 0
        for i in range(len(self.aspects_list)):
            if i == 12:
                nl = 100
                # if len(self.aspects_list) > 24:
                #     line = -1 * ( len(self.aspects_list) - 24) * 14
                # else:
                #     line = 0

                # temporary:
                line = 0

            if i == 24:
                nl = 200
                # if len(self.aspects_list) > 36:
                #     line = -1 * ( len(self.aspects_list) - 36) * 14
                # else:
                #     line = 0
                line = 0

            if i == 36:
                nl = 300
                if len(self.aspects_list) > 48:
                    line = -1 * (len(self.aspects_list) - 48) * 14
                else:
                    line = 0
            out += '<g transform="translate(%s,%s)">' % (nl, line)
            # first planet symbol
            out += '<use transform="scale(0.4)" x="0" y="3" xlink:href="#%s" />\n' % (
                self.planets_settings[self.aspects_list[i]['p1']]['name'])
            # aspect symbol
            out += '<use  x="15" y="0" xlink:href="#orb%s" />\n' % (
                self.aspects_settings[self.aspects_list[i]['aid']]['degree'])
            # second planet symbol
            out += '<g transform="translate(30,0)">'
            out += '<use transform="scale(0.4)" x="0" y="3" xlink:href="#%s" />\n' % (
                self.planets_settings[self.aspects_list[i]['p2']]['name'])
            out += '</g>'
            # difference in degrees
            out += '<text y="8" x="45" style="fill:%s; font-size: 10px;">%s</text>' % (
                self.colors_settings['paper_0'],
                self.dec2deg(self.aspects_list[i]['orbit']))
            # line
            out += '</g>'
            line = line + 14
        out += '</g>'
        return out

    def makeElements(self, r):
        total = self.fire + self.earth + self.air + self.water
        pf = int(round(100*self.fire/total))
        pe = int(round(100*self.earth/total))
        pa = int(round(100*self.air/total))
        pw = int(round(100*self.water/total))
        out = '<g transform="translate(-30,79)">\n'
        out = out + '<text y="0" style="fill:#ff6600; font-size: 10px;">' + \
            self.language_settings['fire']+'  '+str(pf)+'%</text>\n'
        out = out + '<text y="12" style="fill:#6a2d04; font-size: 10px;">' + \
            self.language_settings['earth']+' '+str(pe)+'%</text>\n'
        out = out + '<text y="24" style="fill:#6f76d1; font-size: 10px;">' + \
            self.language_settings['air']+'   '+str(pa)+'%</text>\n'
        out = out + '<text y="36" style="fill:#630e73; font-size: 10px;">' + \
            self.language_settings['water']+' '+str(pw)+'%</text>\n'
        out = out + '</g>\n'
        return out

    def makePlanetGrid(self):
        out = '<g transform="translate(500,-20)">'

        # loop over all planets
        li = 10
        offset = 0

        out = out + '<g transform="translate(140, -15)">'
        out = out + \
            f'<text text-anchor="end" style="fill:{self.colors_settings["paper_0"]}; font-size: 14px;">{self.language_settings["planets_and_house"]} {self.name}:</text>'
        out = out + '</g>\n'

        for i in range(len(self.planets_settings)):

            # Guarda qui !!
            if i == 27:
                li = 10
                offset = -120
            if self.planets_settings[i]['visible'] == 1:
                # start of line
                out = out + '<g transform="translate(%s,%s)">' % (offset, li)
                # planet text
                out = out + '<text text-anchor="end" style="fill:%s; font-size: 10px;">%s</text>' % (
                    self.colors_settings['paper_0'], self.planets_settings[i]['label'])
                # planet symbol
                out = out + \
                    '<g transform="translate(5,-8)"><use transform="scale(0.4)" xlink:href="#' + \
                    self.planets_settings[i]['name']+'" /></g>'
                # planet degree
                out = out + '<text text-anchor="start" x="19" style="fill:%s; font-size: 10px;">%s</text>' % (
                    self.colors_settings['paper_0'], self.dec2deg(self.planets_degree[i]))
                # zodiac
                out = out + '<g transform="translate(60,-8)"><use transform="scale(0.3)" xlink:href="#' + \
                    self.zodiac[self.planets_sign[i]]+'" /></g>'
                # planet retrograde
                if self.planets_retrograde[i]:
                    out = out + \
                        '<g transform="translate(74,-6)"><use transform="scale(.5)" xlink:href="#retrograde" /></g>'

                # end of line
                out = out + '</g>\n'
                # offset between lines
                li = li + 14

        # ----------

        if self.type == "Transit" or self.type == "Composite":

            if self.type == "Transit":
                out = out + '<g transform="translate(320, -15)">'
                out = out + \
                    f'<text text-anchor="end" style="fill:{self.colors_settings["paper_0"]}; font-size: 14px;">{self.t_name}:</text>'
            else:
                out = out + '<g transform="translate(380, -15)">'
                out = out + \
                    f'<text text-anchor="end" style="fill:{self.colors_settings["paper_0"]}; font-size: 14px;">{self.language_settings["planets_and_house"]} {self.t_user.name}:</text>'
            out = out + '</g>\n'

            t_li = 10
            t_offset = 250

            for i in range(len(self.planets_settings)):

                # Guarda qui !!
                if i == 27:
                    t_li = 10
                    t_offset = -120
                if self.planets_settings[i]['visible'] == 1:
                    # start of line
                    out = out + \
                        '<g transform="translate(%s,%s)">' % (t_offset, t_li)
                    # planet text
                    out = out + '<text text-anchor="end" style="fill:%s; font-size: 10px;">%s</text>' % (
                        self.colors_settings['paper_0'], self.planets_settings[i]['label'])
                    # planet symbol
                    out = out + \
                        '<g transform="translate(5,-8)"><use transform="scale(0.4)" xlink:href="#' + \
                        self.planets_settings[i]['name']+'" /></g>'
                    # planet degree
                    out = out + '<text text-anchor="start" x="19" style="fill:%s; font-size: 10px;">%s</text>' % (
                        self.colors_settings['paper_0'], self.dec2deg(self.t_planets_degree[i]))
                    # zodiac
                    out = out + '<g transform="translate(60,-8)"><use transform="scale(0.3)" xlink:href="#' + \
                        self.zodiac[self.t_planets_sign[i]]+'" /></g>'
                    # planet retrograde
                    if self.t_planets_retrograde[i]:
                        out = out + \
                            '<g transform="translate(74,-6)"><use transform="scale(.5)" xlink:href="#retrograde" /></g>'

                    # end of line
                    out = out + '</g>\n'
                    # offset between lines
                    t_li = t_li + 14

        # ----------

        out = out + '</g>\n'

        return out

    def makeHousesGrid(self):

        out = '<g transform="translate(600,-20)">'
        li = 10
        for i in range(12):
            if i < 9:
                cusp = '&#160;&#160;'+str(i+1)
            else:
                cusp = str(i+1)
            out += '<g transform="translate(0,'+str(li)+')">'
            out += '<text text-anchor="end" x="40" style="fill:%s; font-size: 10px;">%s %s:</text>' % (
                self.colors_settings['paper_0'], self.language_settings['cusp'], cusp)
            out += '<g transform="translate(40,-8)"><use transform="scale(0.3)" xlink:href="#' + \
                self.zodiac[self.houses_sign[i]]+'" /></g>'
            out += '<text x="53" style="fill:%s; font-size: 10px;"> %s</text>' % (
                self.colors_settings['paper_0'], self.dec2deg(self.houses_list[i]["position"]))
            out += '</g>\n'
            li = li + 14
        out += '</g>\n'

        # ----------

        if self.type == "Composite":
            out += '<g transform="translate(840, -20)">'
            li = 10
            for i in range(12):
                if i < 9:
                    cusp = '&#160;&#160;'+str(i+1)
                else:
                    cusp = str(i+1)
                out += '<g transform="translate(0,'+str(li)+')">'
                out += '<text text-anchor="end" x="40" style="fill:%s; font-size: 10px;">%s %s:</text>' % (
                    self.colors_settings['paper_0'], self.language_settings['cusp'], cusp)
                out += '<g transform="translate(40,-8)"><use transform="scale(0.3)" xlink:href="#' + \
                    self.zodiac[self.t_houses_sign[i]]+'" /></g>'
                out += '<text x="53" style="fill:%s; font-size: 10px;"> %s</text>' % (
                    self.colors_settings['paper_0'], self.dec2deg(self.t_houses_list[i]["position"]))
                out += '</g>\n'
                li = li + 14
            out += '</g>\n'

        return out

    def set_output_directory(self, dir_path):
        """
        Sets the output direcotry and returns it's path.
        """
        self.output_directory = dir_path
        dir_string = f"Output direcotry set to: {self.output_directory}"
        return (print(dir_string))


if __name__ == "__main__":

    first = kr.KrInstance("Jack", 1990, 6, 15, 15, 15, "Roma")
    second = kr.KrInstance("Jane", 1991, 10, 25, 21, 00, "Roma")

    name = MakeSvgInstance(first, chart_type="Natal", second_obj=second)
    name.output_directory = os.path.expanduser("~")
    template = name.makeTemplate()
    name.makeSVG()
    # print(len(name.aspects_list))
    kr.print_settings_path()
    print(template)
